@@ -5,8 +5,6 @@
  *
  * Part:        Layer4 asynchronous primitives.
  *
- * Version:     $Id: layer4.c,v 1.1.16 2009/02/14 03:25:07 acassen Exp $
- *
  * Authors:     Alexandre Cassen, <acassen@linux-vs.org>
  *
  *              This program is distributed in the hope that it will be useful,
@@ -19,20 +17,18 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #include "config.h"
 
+#include <fcntl.h>
+
 /* keepalived include */
 #include "utils.h"
 
 /* genhash includes */
 #include "include/layer4.h"
-#include "include/main.h"
-#include "include/sock.h"
-#include "include/http.h"
-#include "include/ssl.h"
 
 static enum connect_result
 tcp_connect(int fd, REQ * req_obj)
@@ -111,7 +107,7 @@ tcp_socket_state(thread_t * thread, int (*func) (thread_t *))
 	if (thread->type == THREAD_WRITE_TIMEOUT) {
 		DBG("TCP connection timeout to [%s]:%d.\n",
 		    req->ipaddress, ntohs(req->addr_port));
-		close(thread->u.fd);
+		thread_close_fd(thread);
 		return connect_timeout;
 	}
 
@@ -125,7 +121,7 @@ tcp_socket_state(thread_t * thread, int (*func) (thread_t *))
 	if (ret) {
 		DBG("TCP connection failed to [%s]:%d.\n",
 		    req->ipaddress, ntohs(req->addr_port));
-		close(thread->u.fd);
+		thread_close_fd(thread);
 		return connect_error;
 	}
 
@@ -207,6 +203,7 @@ tcp_check_thread(thread_t * thread)
 				sock_obj->lock = 0;
 				thread_add_event(thread->master,
 						 http_request_thread, sock_obj, 0);
+				thread_del_write(thread);
 			} else {
 				DBG("Connection trouble to: [%s]:%d.\n",
 				    req->ipaddress,
