@@ -19,13 +19,14 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #include "config.h"
 
 #include <errno.h>
 #include <stdint.h>
+#include <arpa/inet.h>
 
 #include "check_parser.h"
 #include "check_data.h"
@@ -35,20 +36,30 @@
 #include "main.h"
 #include "logger.h"
 #include "parser.h"
-#include "memory.h"
 #include "utils.h"
 #include "ipwrapper.h"
 #if defined _WITH_VRRP_
 #include "vrrp_parser.h"
 #endif
+#if defined _WITH_BFD_
+#include "bfd_parser.h"
+#endif
+#include "libipvs.h"
+
+/* List of valid schedulers */
+char *lvs_schedulers[] =
+	{"rr", "wrr", "lc", "wlc", "lblc", "sh", "mh", "dh", "fo", "ovf", "lblcr", "sed", "nq", NULL};
 
 /* SSL handlers */
 static void
-ssl_handler(__attribute__((unused)) vector_t *strvec)
+ssl_handler(vector_t *strvec)
 {
+	if (!strvec)
+		return;
+
 	if (check_data->ssl) {
 		free_ssl();
-		log_message(LOG_INFO, "SSL context already specified - replacing");
+		report_config_error(CONFIG_GENERAL_ERROR, "SSL context already specified - replacing");
 	}
 	check_data->ssl = alloc_ssl();
 }
@@ -56,7 +67,7 @@ static void
 sslpass_handler(vector_t *strvec)
 {
 	if (check_data->ssl->password) {
-		log_message(LOG_INFO, "SSL password already specified - replacing");
+		report_config_error(CONFIG_GENERAL_ERROR, "SSL password already specified - replacing");
 		FREE(check_data->ssl->password);
 	}
 	check_data->ssl->password = set_value(strvec);
@@ -65,7 +76,7 @@ static void
 sslca_handler(vector_t *strvec)
 {
 	if (check_data->ssl->cafile) {
-		log_message(LOG_INFO, "SSL cafile already specified - replacing");
+		report_config_error(CONFIG_GENERAL_ERROR, "SSL cafile already specified - replacing");
 		FREE(check_data->ssl->cafile);
 	}
 	check_data->ssl->cafile = set_value(strvec);
@@ -74,7 +85,7 @@ static void
 sslcert_handler(vector_t *strvec)
 {
 	if (check_data->ssl->certfile) {
-		log_message(LOG_INFO, "SSL certfile already specified - replacing");
+		report_config_error(CONFIG_GENERAL_ERROR, "SSL certfile already specified - replacing");
 		FREE(check_data->ssl->certfile);
 	}
 	check_data->ssl->certfile = set_value(strvec);
@@ -83,7 +94,7 @@ static void
 sslkey_handler(vector_t *strvec)
 {
 	if (check_data->ssl->keyfile) {
-		log_message(LOG_INFO, "SSL keyfile already specified - replacing");
+		report_config_error(CONFIG_GENERAL_ERROR, "SSL keyfile already specified - replacing");
 		FREE(check_data->ssl->keyfile);
 	}
 	check_data->ssl->keyfile = set_value(strvec);
@@ -95,30 +106,86 @@ vsg_handler(vector_t *strvec)
 {
 	virtual_server_group_t *vsg;
 
+	if (!strvec)
+		return;
+
 	/* Fetch queued vsg */
 	alloc_vsg(strvec_slot(strvec, 1));
-	alloc_value_block(alloc_vsg_entry);
+	alloc_value_block(alloc_vsg_entry, strvec_slot(strvec, 0));
 
 	/* Ensure the virtual server group has some configuration */
 	vsg = LIST_TAIL_DATA(check_data->vs_group);
 	if (LIST_ISEMPTY(vsg->vfwmark) && LIST_ISEMPTY(vsg->addr_range)) {
-		log_message(LOG_INFO, "virtual server group %s has no entries - removing", vsg->gname);
+		report_config_error(CONFIG_GENERAL_ERROR, "virtual server group %s has no entries - removing", vsg->gname);
 		free_list_element(check_data->vs_group, check_data->vs_group->tail);
 	}
 }
 static void
 vs_handler(vector_t *strvec)
 {
-	alloc_vs(strvec_slot(strvec, 1), strvec_slot(strvec, 2));
+	global_data->have_checker_config = true;
+
+	/* If we are not in the checker process, we don't want any more info */
+	if (!strvec)
+		return;
+
+	alloc_vs(strvec_slot(strvec, 1), vector_size(strvec) >= 3 ? strvec_slot(strvec, 2) : NULL);
 }
 static void
 vs_end_handler(void)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
+	real_server_t *rs;
+	element e;
 
-	/* I'm not sure how the following can happen so long as we have a real server */
-	if (!vs->af)
-		vs->af = AF_INET;
+	/* If the real (sorry) server uses tunnel forwarding, the address family
+	 * does not have to match the address family of the virtual server */
+	if (vs->s_svr
+#if HAVE_DECL_IPVS_DEST_ATTR_ADDR_FAMILY
+		      && vs->s_svr->forwarding_method != IP_VS_CONN_F_TUNNEL
+#endif
+									    )
+	{
+		if (vs->af == AF_UNSPEC)
+			vs->af = vs->s_svr->addr.ss_family;
+		else if (vs->af != vs->s_svr->addr.ss_family) {
+			report_config_error(CONFIG_GENERAL_ERROR, "Address family of virtual server and sorry server %s don't match - skipping sorry server.", inet_sockaddrtos(&vs->s_svr->addr));
+			FREE(vs->s_svr);
+			vs->s_svr = NULL;
+		}
+	}
+
+	if (vs->af == AF_UNSPEC) {
+		/* This only occurs if the virtual server uses a fwmark, and all the
+		 * real/sorry servers are tunnelled.
+		 *
+		 * Maintain backward compatibility. Prior to the commit following 17fa4a3c
+		 * the address family of the virtual server was set from any of its
+		 * real or sorry servers, even if they were tunnelled. However, all the real
+		 * and sorry servers had to be the same address family, even if tunnelled,
+		 * so only set the address family from the tunnelled real/sorry servers
+		 * if all the real/sorry servers are of the same address family. */
+		if (vs->s_svr)
+			vs->af = vs->s_svr->addr.ss_family;
+
+		if (!LIST_ISEMPTY(vs->rs)) {
+			for (e = LIST_HEAD(vs->rs); e; ELEMENT_NEXT(e)) {
+				rs = ELEMENT_DATA(e);
+				if (vs->af == AF_UNSPEC)
+					vs->af = rs->addr.ss_family;
+				else if (vs->af != rs->addr.ss_family) {
+					vs->af = AF_UNSPEC;
+					break;
+				}
+			}
+		}
+
+		if (vs->af == AF_UNSPEC) {
+			/* We have a mixture of IPv4 and IPv6 tunnelled real/sorry servers.
+			 * Default to IPv4. */
+			vs->af = AF_INET;
+		}
+	}
 }
 static void
 ip_family_handler(vector_t *strvec)
@@ -130,20 +197,20 @@ ip_family_handler(vector_t *strvec)
 		af = AF_INET;
 	else if (!strcmp(strvec_slot(strvec, 1), "inet6")) {
 #ifndef LIBIPVS_USE_NL
-		log_message(LOG_INFO, "IPVS with IPv6 is not supported by this build");
-		skip_block();
+		report_config_error(CONFIG_GENERAL_ERROR, "IPVS with IPv6 is not supported by this build");
+		skip_block(false);
 		return;
 #endif
 		af = AF_INET6;
 	}
 	else {
-		log_message(LOG_INFO, "unknown address family %s", FMT_STR_VSLOT(strvec, 1));
+		report_config_error(CONFIG_GENERAL_ERROR, "unknown address family %s", FMT_STR_VSLOT(strvec, 1));
 		return;
 	}
 
 	if (vs->af != AF_UNSPEC &&
 	    af != vs->af) {
-		log_message(LOG_INFO, "Virtual server specified family %s conflicts with server family", FMT_STR_VSLOT(strvec, 1));
+		report_config_error(CONFIG_GENERAL_ERROR, "Virtual server specified family %s conflicts with server family", FMT_STR_VSLOT(strvec, 1));
 		return;
 	}
 
@@ -153,47 +220,63 @@ static void
 vs_delay_handler(vector_t *strvec)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
-	vs->delay_loop = read_timer(strvec);
+	unsigned long delay;
+
+	if (read_timer(strvec, 1, &delay, 1, 0, true))
+		vs->delay_loop = delay;
+	else
+		report_config_error(CONFIG_GENERAL_ERROR, "virtual server delay loop '%s' invalid - ignoring", FMT_STR_VSLOT(strvec, 1));
 }
 static void
 vs_delay_before_retry_handler(vector_t *strvec)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
-	vs->delay_before_retry = read_timer(strvec);
+	unsigned long delay;
+
+	if (read_timer(strvec, 1, &delay, 0, 0, true))
+		vs->delay_before_retry = delay;
+	else
+		report_config_error(CONFIG_GENERAL_ERROR, "virtual server delay before retry '%s' invalid - ignoring", FMT_STR_VSLOT(strvec, 1));
 }
 static void
 vs_retry_handler(vector_t *strvec)
 {
-	unsigned long retry;
-	char *endptr;
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
+	unsigned retry;
 
-	errno = 0;
-	retry = strtoul(strvec_slot(strvec, 1), &endptr, 10);
-	if (errno || *endptr || retry > UINT32_MAX || retry == 0) {
-		log_message(LOG_INFO, "retry value invalid - %s", FMT_STR_VSLOT(strvec, 1));
+	if (!read_unsigned_strvec(strvec, 1, &retry, 1, UINT32_MAX, false)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "retry value invalid - %s", FMT_STR_VSLOT(strvec, 1));
 		return;
 	}
-	vs->retry = (unsigned)retry;
+	vs->retry = retry;
 }
 static void
 vs_warmup_handler(vector_t *strvec)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
-	vs->warmup = read_timer(strvec);
+	unsigned long delay;
+
+	if (read_timer(strvec, 1, &delay, 0, 0, true))
+		vs->warmup = delay;
+	else
+		report_config_error(CONFIG_GENERAL_ERROR, "virtual server warmup '%s' invalid - ignoring", FMT_STR_VSLOT(strvec, 1));
 }
 static void
 lbalgo_handler(vector_t *strvec)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
 	char *str = strvec_slot(strvec, 1);
-	size_t size = sizeof (vs->sched);
-	size_t str_len = strlen(str);
+	int i;
 
-	if (size > str_len)
-		size = str_len;
+	/* Check valid scheduler name */
+	for (i = 0; lvs_schedulers[i] && strcmp(str, lvs_schedulers[i]); i++);
 
-	memcpy(vs->sched, str, size);
+	if (!lvs_schedulers[i] || strlen(str) >= sizeof(vs->sched)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "Invalid lvs_scheduler '%s' - ignoring", FMT_STR_VSLOT(strvec, 1));
+		return;
+	}
+
+	strcpy(vs->sched, str);
 }
 
 static void
@@ -223,8 +306,16 @@ lbflags_handler(vector_t *strvec)
 		if (!strcmp(str, "sh-fallback"))
 			vs->flags |= IP_VS_SVC_F_SCHED_SH_FALLBACK;
 	}
+	else if (!strcmp(vs->sched , "mh") )
+	{
+		/* mh-port and mh-fallback flags are relevant for mh scheduler only */
+		if (!strcmp(str, "mh-port")  )
+			vs->flags |= IP_VS_SVC_F_SCHED_MH_PORT;
+		if (!strcmp(str, "mh-fallback"))
+			vs->flags |= IP_VS_SVC_F_SCHED_MH_FALLBACK;
+	}
 	else
-		log_message(LOG_INFO, "%s only applies to sh scheduler - ignoring", str);
+		report_config_error(CONFIG_GENERAL_ERROR, "%s only applies to sh scheduler - ignoring", str);
 #endif
 }
 
@@ -241,24 +332,21 @@ forwarding_handler(vector_t *strvec)
 	else if (!strcmp(str, "TUN"))
 		vs->forwarding_method = IP_VS_CONN_F_TUNNEL;
 	else
-		log_message(LOG_INFO, "PARSER : unknown [%s] routing method.", str);
+		report_config_error(CONFIG_GENERAL_ERROR, "PARSER : unknown [%s] routing method.", str);
 }
 static void
 pto_handler(vector_t *strvec)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
-	char *endptr;
-	unsigned long timeout;
+	unsigned timeout;
 
 	if (vector_size(strvec) < 2) {
 		vs->persistence_timeout = IPVS_SVC_PERSISTENT_TIMEOUT;
 		return;
 	}
 
-	errno = 0;
-	timeout = strtoul(strvec_slot(strvec, 1), &endptr, 10);
-	if (errno || *endptr || timeout > UINT32_MAX || timeout == 0) {
-		log_message(LOG_INFO, "persistence_timeout invalid");
+	if (!read_unsigned_strvec(strvec, 1, &timeout, 1, UINT32_MAX, false)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "persistence_timeout invalid");
 		return;
 	}
 
@@ -281,22 +369,21 @@ pgr_handler(vector_t *strvec)
 {
 	struct in_addr addr;
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
-	char *endptr;
 	uint16_t af = vs->af;
+	unsigned granularity;
 
 	if (af == AF_UNSPEC)
 		af = strchr(strvec_slot(strvec, 1), '.') ? AF_INET : AF_INET6;
 
 	if (af == AF_INET6) {
-		vs->persistence_granularity = (uint32_t)strtoul(strvec_slot(strvec, 1), &endptr, 10);
-		if (*endptr || vs->persistence_granularity < 1 || vs->persistence_granularity > 128) {
-			log_message(LOG_INFO, "Invalid IPv6 persistence_granularity specified - %s", FMT_STR_VSLOT(strvec, 1));
-			vs->persistence_granularity = 0;
+		if (!read_unsigned_strvec(strvec, 1, &granularity, 1, 128, false)) {
+			report_config_error(CONFIG_GENERAL_ERROR, "Invalid IPv6 persistence_granularity specified - %s", FMT_STR_VSLOT(strvec, 1));
 			return;
 		}
+		vs->persistence_granularity = granularity;
 	} else {
-		if (inet_aton(strvec_slot(strvec, 1), &addr)) {
-			log_message(LOG_INFO, "Invalid IPv4 persistence_granularity specified - %s", FMT_STR_VSLOT(strvec, 1));
+		if (!inet_aton(strvec_slot(strvec, 1), &addr)) {
+			report_config_error(CONFIG_GENERAL_ERROR, "Invalid IPv4 persistence_granularity specified - %s", FMT_STR_VSLOT(strvec, 1));
 			return;
 		}
 
@@ -305,7 +392,7 @@ pgr_handler(vector_t *strvec)
 		while (!(haddr & 1))
 			haddr = (haddr >> 1) | 0x80000000;
 		if (haddr != 0xffffffff) {
-			log_message(LOG_INFO, "IPv4 persistence_granularity netmask is not solid - %s", FMT_STR_VSLOT(strvec, 1));
+			report_config_error(CONFIG_GENERAL_ERROR, "IPv4 persistence_granularity netmask is not solid - %s", FMT_STR_VSLOT(strvec, 1));
 			return;
 		}
 
@@ -323,14 +410,15 @@ proto_handler(vector_t *strvec)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
 	char *str = strvec_slot(strvec, 1);
-	if (!strcmp(str, "TCP"))
+
+	if (!strcasecmp(str, "TCP"))
 		vs->service_type = IPPROTO_TCP;
-	else if (!strcmp(str, "SCTP"))
+	else if (!strcasecmp(str, "SCTP"))
 		vs->service_type = IPPROTO_SCTP;
-	else if (!strcmp(str, "UDP"))
+	else if (!strcasecmp(str, "UDP"))
 		vs->service_type = IPPROTO_UDP;
 	else
-		log_message(LOG_INFO, "Unknown protocol %s - ignoring", str);
+		report_config_error(CONFIG_GENERAL_ERROR, "Unknown protocol %s - ignoring", str);
 }
 static void
 hasuspend_handler(__attribute__((unused)) vector_t *strvec)
@@ -339,6 +427,22 @@ hasuspend_handler(__attribute__((unused)) vector_t *strvec)
 	vs->ha_suspend = true;
 }
 
+static void
+vs_smtp_alert_handler(vector_t *strvec)
+{
+	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
+	int res = true;
+
+	if (vector_size(strvec) >= 2) {
+		res = check_true_false(strvec_slot(strvec, 1));
+		if (res == -1) {
+			report_config_error(CONFIG_GENERAL_ERROR, "Invalid virtual_server smtp_alert parameter %s", FMT_STR_VSLOT(strvec, 1));
+			return;
+		}
+	}
+	vs->smtp_alert = res;
+}
+
 static void
 vs_virtualhost_handler(vector_t *strvec)
 {
@@ -358,7 +462,7 @@ svr_forwarding_handler(real_server_t *rs, vector_t *strvec)
 	else if (!strcmp(str, "TUN"))
 		rs->forwarding_method = IP_VS_CONN_F_TUNNEL;
 	else
-		log_message(LOG_INFO, "PARSER : unknown [%s] routing method for real server.", str);
+		report_config_error(CONFIG_GENERAL_ERROR, "PARSER : unknown [%s] routing method for real server.", str);
 }
 /* Sorry Servers handlers */
 static void
@@ -373,7 +477,7 @@ ssvri_handler(__attribute__((unused)) vector_t *strvec)
 	if (vs->s_svr)
 		vs->s_svr->inhibit = true;
 	else
-		log_message(LOG_ERR, "Ignoring sorry_server inhibit used before or without sorry_server");
+		report_config_error(CONFIG_GENERAL_ERROR, "Ignoring sorry_server inhibit used before or without sorry_server");
 }
 static void
 ss_forwarding_handler(vector_t *strvec)
@@ -383,7 +487,7 @@ ss_forwarding_handler(vector_t *strvec)
 	if (vs->s_svr)
 		svr_forwarding_handler(vs->s_svr, strvec);
 	else
-		log_message(LOG_ERR, "sorry_server forwarding used without sorry_server");
+		report_config_error(CONFIG_GENERAL_ERROR, "sorry_server forwarding used without sorry_server");
 }
 
 /* Real Servers handlers */
@@ -393,12 +497,48 @@ rs_handler(vector_t *strvec)
 	alloc_rs(strvec_slot(strvec, 1), strvec_slot(strvec, 2));
 }
 static void
+rs_end_handler(void)
+{
+	virtual_server_t *vs;
+	real_server_t *rs;
+
+	if (LIST_ISEMPTY(check_data->vs))
+		return;
+
+	vs = LIST_TAIL_DATA(check_data->vs);
+
+	if (LIST_ISEMPTY(vs->rs))
+		return;
+
+	rs = LIST_TAIL_DATA(vs->rs);
+
+	/* For tunnelled forwarding, the address families don't have to be the same, so
+	 * long as the kernel supports IPVS_DEST_ATTR_ADDR_FAMILY */
+#if HAVE_DECL_IPVS_DEST_ATTR_ADDR_FAMILY
+	if (rs->forwarding_method != IP_VS_CONN_F_TUNNEL)
+#endif
+	{
+		if (vs->af == AF_UNSPEC)
+			vs->af = rs->addr.ss_family;
+		else if (vs->af != rs->addr.ss_family) {
+			report_config_error(CONFIG_GENERAL_ERROR, "Address family of virtual server and real server %s don't match - skipping real server.", inet_sockaddrtos(&rs->addr));
+			free_list_element(vs->rs, vs->rs->tail);
+		}
+	}
+}
+static void
 rs_weight_handler(vector_t *strvec)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
 	real_server_t *rs = LIST_TAIL_DATA(vs->rs);
-	rs->weight = atoi(strvec_slot(strvec, 1));
-	rs->iweight = rs->weight;
+	unsigned weight;
+
+	if (!read_unsigned_strvec(strvec, 1, &weight, 1, 65535, true)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "Real server weight %s is outside range 1-65535", FMT_STR_VSLOT(strvec, 1));
+		return;
+	}
+	rs->weight = weight;
+	rs->iweight = weight;
 }
 static void
 rs_forwarding_handler(vector_t *strvec)
@@ -413,14 +553,26 @@ uthreshold_handler(vector_t *strvec)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
 	real_server_t *rs = LIST_TAIL_DATA(vs->rs);
-	rs->u_threshold = (uint32_t)strtoul(strvec_slot(strvec, 1), NULL, 10);
+	unsigned threshold;
+
+	if (!read_unsigned_strvec(strvec, 1, &threshold, 0, UINT_MAX, true)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "Invalid real_server uthreshold '%s' - ignoring", FMT_STR_VSLOT(strvec, 1));
+		return;
+	}
+	rs->u_threshold = threshold;
 }
 static void
 lthreshold_handler(vector_t *strvec)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
 	real_server_t *rs = LIST_TAIL_DATA(vs->rs);
-	rs->l_threshold = (uint32_t)strtoul(strvec_slot(strvec, 1), NULL, 10);
+	unsigned threshold;
+
+	if (!read_unsigned_strvec(strvec, 1, &threshold, 0, UINT_MAX, true)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "Invalid real_server lthreshold '%s' - ignoring", FMT_STR_VSLOT(strvec, 1));
+		return;
+	}
+	rs->l_threshold = threshold;
 }
 static void
 vs_inhibit_handler(__attribute__((unused)) vector_t *strvec)
@@ -429,9 +581,9 @@ vs_inhibit_handler(__attribute__((unused)) vector_t *strvec)
 	vs->inhibit = true;
 }
 static inline notify_script_t*
-set_check_notify_script(vector_t *strvec, const char *type)
+set_check_notify_script(__attribute__((unused)) vector_t *strvec, const char *type)
 {
-	return notify_script_init(strvec, type, global_data->script_security);
+	return notify_script_init(0, type);
 }
 static void
 notify_up_handler(vector_t *strvec)
@@ -439,7 +591,7 @@ notify_up_handler(vector_t *strvec)
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
 	real_server_t *rs = LIST_TAIL_DATA(vs->rs);
 	if (rs->notify_up) {
-		log_message(LOG_INFO, "(%s): notify_up script already specified - ignoring %s", vs->vsgname, FMT_STR_VSLOT(strvec,1));
+		report_config_error(CONFIG_GENERAL_ERROR, "(%s) notify_up script already specified - ignoring %s", vs->vsgname, FMT_STR_VSLOT(strvec,1));
 		return;
 	}
 	rs->notify_up = set_check_notify_script(strvec, "notify");
@@ -450,7 +602,7 @@ notify_down_handler(vector_t *strvec)
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
 	real_server_t *rs = LIST_TAIL_DATA(vs->rs);
 	if (rs->notify_down) {
-		log_message(LOG_INFO, "(%s): notify_down script already specified - ignoring %s", vs->vsgname, FMT_STR_VSLOT(strvec,1));
+		report_config_error(CONFIG_GENERAL_ERROR, "(%s) notify_down script already specified - ignoring %s", vs->vsgname, FMT_STR_VSLOT(strvec,1));
 		return;
 	}
 	rs->notify_down = set_check_notify_script(strvec, "notify");
@@ -460,27 +612,34 @@ rs_delay_handler(vector_t *strvec)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
 	real_server_t *rs = LIST_TAIL_DATA(vs->rs);
-	rs->delay_loop = read_timer(strvec);
+	unsigned long delay;
+
+	if (read_timer(strvec, 1, &delay, 1, 0, true))
+		rs->delay_loop = delay;
+	else
+		report_config_error(CONFIG_GENERAL_ERROR, "real server delay_loop '%s' invalid - ignoring", FMT_STR_VSLOT(strvec, 1));
 }
 static void
 rs_delay_before_retry_handler(vector_t *strvec)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
 	real_server_t *rs = LIST_TAIL_DATA(vs->rs);
-	rs->delay_before_retry = read_timer(strvec);
+	unsigned long delay;
+
+	if (read_timer(strvec, 1, &delay, 0, 0, true))
+		rs->delay_before_retry = delay;
+	else
+		report_config_error(CONFIG_GENERAL_ERROR, "real server delay_before_retry '%s' invalid - ignoring", FMT_STR_VSLOT(strvec, 1));
 }
 static void
 rs_retry_handler(vector_t *strvec)
 {
-	unsigned long retry;
-	char *endptr;
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
 	real_server_t *rs = LIST_TAIL_DATA(vs->rs);
+	unsigned retry;
 
-	errno = 0;
-	retry = strtoul(strvec_slot(strvec, 1), &endptr, 10);
-	if (errno || *endptr || retry > UINT32_MAX || retry == 0) {
-		log_message(LOG_INFO, "retry value invalid - %s", FMT_STR_VSLOT(strvec, 1));
+	if (!read_unsigned_strvec(strvec, 1, &retry, 1, UINT32_MAX, false)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "retry value invalid - %s", FMT_STR_VSLOT(strvec, 1));
 		return;
 	}
 	rs->retry = (unsigned)retry;
@@ -490,7 +649,12 @@ rs_warmup_handler(vector_t *strvec)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
 	real_server_t *rs = LIST_TAIL_DATA(vs->rs);
-	rs->warmup = read_timer(strvec);
+	unsigned long delay;
+
+	if (read_timer(strvec, 1, &delay, 0, 0, true))
+		rs->warmup = delay;
+	else
+		report_config_error(CONFIG_GENERAL_ERROR, "real server warmup '%s' invalid - ignoring", FMT_STR_VSLOT(strvec, 1));
 }
 static void
 rs_inhibit_handler(vector_t *strvec)
@@ -502,7 +666,7 @@ rs_inhibit_handler(vector_t *strvec)
 	if (vector_size(strvec) >= 2) {
 		res = check_true_false(strvec_slot(strvec, 1));
 		if (res == -1) {
-			log_message(LOG_INFO, "Invalid inhibit_on_failure parameter %s", FMT_STR_VSLOT(strvec, 1));
+			report_config_error(CONFIG_GENERAL_ERROR, "Invalid inhibit_on_failure parameter %s", FMT_STR_VSLOT(strvec, 1));
 			return;
 		}
 	}
@@ -518,13 +682,29 @@ rs_alpha_handler(vector_t *strvec)
 	if (vector_size(strvec) >= 2) {
 		res = check_true_false(strvec_slot(strvec, 1));
 		if (res == -1) {
-			log_message(LOG_INFO, "Invalid alpha parameter %s", FMT_STR_VSLOT(strvec, 1));
+			report_config_error(CONFIG_GENERAL_ERROR, "Invalid alpha parameter %s", FMT_STR_VSLOT(strvec, 1));
 			return;
 		}
 	}
 	rs->alpha = res;
 }
 static void
+rs_smtp_alert_handler(vector_t *strvec)
+{
+	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
+	real_server_t *rs = LIST_TAIL_DATA(vs->rs);
+	int res = true;
+
+	if (vector_size(strvec) >= 2) {
+		res = check_true_false(strvec_slot(strvec, 1));
+		if (res == -1) {
+			report_config_error(CONFIG_GENERAL_ERROR, "Invalid real_server smtp_alert parameter %s", FMT_STR_VSLOT(strvec, 1));
+			return;
+		}
+	}
+	rs->smtp_alert = res;
+}
+static void
 rs_virtualhost_handler(vector_t *strvec)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
@@ -548,7 +728,7 @@ quorum_up_handler(vector_t *strvec)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
 	if (vs->notify_quorum_up) {
-		log_message(LOG_INFO, "(%s): quorum_up script already specified - ignoring %s", vs->vsgname, FMT_STR_VSLOT(strvec,1));
+		report_config_error(CONFIG_GENERAL_ERROR, "(%s) quorum_up script already specified - ignoring %s", vs->vsgname, FMT_STR_VSLOT(strvec,1));
 		return;
 	}
 	vs->notify_quorum_up = set_check_notify_script(strvec, "quorum");
@@ -558,7 +738,7 @@ quorum_down_handler(vector_t *strvec)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
 	if (vs->notify_quorum_down) {
-		log_message(LOG_INFO, "(%s): quorum_down script already specified - ignoring %s", vs->vsgname, FMT_STR_VSLOT(strvec,1));
+		report_config_error(CONFIG_GENERAL_ERROR, "(%s) quorum_down script already specified - ignoring %s", vs->vsgname, FMT_STR_VSLOT(strvec,1));
 		return;
 	}
 	vs->notify_quorum_down = set_check_notify_script(strvec, "quorum");
@@ -567,26 +747,39 @@ static void
 quorum_handler(vector_t *strvec)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
-	vs->quorum = (unsigned)strtoul(strvec_slot(strvec, 1), NULL, 10);
-	if (vs->quorum < 1) {
-		log_message(LOG_ERR, "Condition not met: Quorum >= 1");
-		log_message(LOG_ERR, "Ignoring requested value %s, using 1 instead",
-		  FMT_STR_VSLOT(strvec, 1));
-		vs->quorum = 1;
+	unsigned quorum;
+
+	if (!read_unsigned_strvec(strvec, 1, &quorum, 1, UINT_MAX, true)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "Quorum %s must be in [1, %u]. Setting to 1.", FMT_STR_VSLOT(strvec, 1), UINT_MAX);
+		quorum = 1;
 	}
+
+	vs->quorum = quorum;
 }
 static void
 hysteresis_handler(vector_t *strvec)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
+	unsigned hysteresis;
 
-	vs->hysteresis = (unsigned)strtoul(strvec_slot(strvec, 1), NULL, 10);
+	if (!read_unsigned_strvec(strvec, 1, &hysteresis, 0, UINT_MAX, true)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "Hysteresis %s must be in [0, %u] - ignoring", FMT_STR_VSLOT(strvec, 1), UINT_MAX);
+		return;
+	}
+
+	vs->hysteresis = hysteresis;
 }
 static void
 vs_weight_handler(vector_t *strvec)
 {
 	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
-	vs->weight = atoi(strvec_slot(strvec, 1));
+	unsigned weight;
+
+	if (!read_unsigned_strvec(strvec, 1, &weight, 1, 65535, true)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "Virtual server weight %s is outside range 1-65535", FMT_STR_VSLOT(strvec, 1));
+		return;
+	}
+	vs->weight = weight;
 }
 
 void
@@ -602,6 +795,7 @@ init_check_keywords(bool active)
 	/* Virtual server mapping */
 	install_keyword_root("virtual_server_group", &vsg_handler, active);
 	install_keyword_root("virtual_server", &vs_handler, active);
+	install_root_end_handler(&vs_end_handler);
 	install_keyword("ip_family", &ip_family_handler);
 	install_keyword("retry", &vs_retry_handler);
 	install_keyword("delay_before_retry", &vs_delay_before_retry_handler);
@@ -631,6 +825,7 @@ init_check_keywords(bool active)
 	install_keyword("persistence_granularity", &pgr_handler);
 	install_keyword("protocol", &proto_handler);
 	install_keyword("ha_suspend", &hasuspend_handler);
+	install_keyword("smtp_alert", &vs_smtp_alert_handler);
 	install_keyword("virtualhost", &vs_virtualhost_handler);
 
 	/* Pool regression detection and handling. */
@@ -660,9 +855,10 @@ init_check_keywords(bool active)
 	install_keyword("delay_before_retry", &rs_delay_before_retry_handler);
 	install_keyword("warmup", &rs_warmup_handler);
 	install_keyword("delay_loop", &rs_delay_handler);
+	install_keyword("smtp_alert", &rs_smtp_alert_handler);
 	install_keyword("virtualhost", &rs_virtualhost_handler);
 
-	install_sublevel_end_handler(&vs_end_handler);
+	install_sublevel_end_handler(&rs_end_handler);
 
 	/* Checkers mapping */
 	install_checkers_keyword();
@@ -673,11 +869,14 @@ vector_t *
 check_init_keywords(void)
 {
 	/* global definitions mapping */
-	init_global_keywords(true);
+	init_global_keywords(reload);
 
 	init_check_keywords(true);
 #ifdef _WITH_VRRP_
 	init_vrrp_keywords(false);
+#endif
+#ifdef _WITH_BFD_
+	init_bfd_keywords(true);
 #endif
 	return keywords;
 }
