@@ -18,24 +18,28 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #include "config.h"
 
+#include <errno.h>
+#include <unistd.h>
+#include <stdio.h>
 #include <ctype.h>
 
 #include "check_smtp.h"
-#include "check_api.h"
 #include "logger.h"
-#include "memory.h"
 #include "ipwrapper.h"
 #include "utils.h"
 #include "parser.h"
-#include "daemon.h"
 #if !HAVE_DECL_SOCK_CLOEXEC
 #include "old_socket.h"
-#include "string.h"
+#endif
+#include "layer4.h"
+#include "smtp.h"
+#ifdef THREAD_DUMP
+#include "scheduler.h"
 #endif
 
 static conn_opts_t* default_co;	/* Default conn_opts for SMTP_CHECK */
@@ -74,18 +78,18 @@ free_smtp_check(void *data)
  * configuration.
  */
 static void
-dump_smtp_check(void *data)
+dump_smtp_check(FILE *fp, void *data)
 {
 	checker_t *checker = data;
 	smtp_checker_t *smtp_checker = checker->data;
 
-	log_message(LOG_INFO, "   Keepalive method = SMTP_CHECK");
-	log_message(LOG_INFO, "   helo = %s", smtp_checker->helo_name);
-	dump_checker_opts(checker);
+	conf_write(fp, "   Keepalive method = SMTP_CHECK");
+	conf_write(fp, "   helo = %s", smtp_checker->helo_name);
+	dump_checker_opts(fp, checker);
 
 	if (smtp_checker->host) {
-		log_message(LOG_INFO,"   Host list");
-		dump_list(smtp_checker->host);
+		conf_write(fp, "   Host list");
+		dump_list(fp, smtp_checker->host);
 	}
 }
 
@@ -161,7 +165,9 @@ smtp_check_end_handler(void)
 	    co->bindto.ss_family != AF_UNSPEC ||
 	    (co->bindto.ss_family == AF_UNSPEC && ((struct sockaddr_in *)&co->bindto)->sin_port) ||
 	    co->bind_if[0] ||
+#ifdef _WITH_SO_MARK_
 	    co->fwmark ||
+#endif
 	    co->connection_to) {
 		/* Set any necessary defaults */
 		if (co->dst.ss_family == AF_UNSPEC) {
@@ -277,9 +283,11 @@ smtp_final(thread_t *thread, int error, const char *format, ...)
 	char error_buff[512];
 	char smtp_buff[542];
 	va_list varg_list;
+	bool checker_was_up;
+	bool rs_was_alive;
 
 	/* Error or no error we should always have to close the socket */
-	close(thread->u.fd);
+	thread_close_fd(thread);
 
 	/* If we're here, an attempt HAS been made already for the current host */
 	checker->retry_it++;
@@ -317,19 +325,23 @@ smtp_final(thread_t *thread, int error, const char *format, ...)
 		 * be noted that smtp_alert makes a copy of the string arguments, so
 		 * we don't have to keep them statically allocated.
 		 */
-		if (checker->is_up) {
-			if (format != NULL) {
-				snprintf(error_buff, sizeof(error_buff), "=> CHECK failed on service : %s <=", format);
-				va_start(varg_list, format);
-				vsnprintf(smtp_buff, sizeof(smtp_buff), error_buff, varg_list);
-				va_end(varg_list);
-			} else {
-				strncpy(smtp_buff, "=> CHECK failed on service <=", sizeof(smtp_buff));
-			}
-
-			smtp_buff[sizeof(smtp_buff) - 1] = '\0';
-			smtp_alert(checker, NULL, NULL, "DOWN", smtp_buff);
+		if (checker->is_up || !checker->has_run) {
+			checker_was_up = checker->is_up;
+			rs_was_alive = checker->rs->alive;
 			update_svr_checker_state(DOWN, checker);
+			if (checker->rs->smtp_alert && checker_was_up &&
+			    (rs_was_alive != checker->rs->alive || !global_data->no_checker_emails)) {
+				if (format != NULL) {
+					snprintf(error_buff, sizeof(error_buff), "=> CHECK failed on service : %s <=", format);
+					va_start(varg_list, format);
+					vsnprintf(smtp_buff, sizeof(smtp_buff), error_buff, varg_list);
+					va_end(varg_list);
+				} else
+					strncpy(smtp_buff, "=> CHECK failed on service <=", sizeof(smtp_buff));
+
+				smtp_buff[sizeof(smtp_buff) - 1] = '\0';
+				smtp_alert(SMTP_MSG_RS, checker, NULL, smtp_buff);
+			}
 		}
 
 		/* Reset everything back to the first host in the list */
@@ -379,7 +391,6 @@ smtp_get_line_cb(thread_t *thread)
 	checker_t *checker = THREAD_ARG(thread);
 	smtp_checker_t *smtp_checker = CHECKER_ARG(checker);
 	conn_opts_t *smtp_host = smtp_checker->host_ptr;
-	int f;
 	unsigned x;
 	ssize_t r;
 
@@ -398,10 +409,6 @@ smtp_get_line_cb(thread_t *thread)
 		smtp_clear_buff(thread);
 	}
 
-	/* Set descriptor non blocking */
-	f = fcntl(thread->u.fd, F_GETFL, 0);
-	fcntl(thread->u.fd, F_SETFL, f | O_NONBLOCK);
-
 	/* read the data */
 	r = read(thread->u.fd, smtp_checker->buff + smtp_checker->buff_ctr,
 		 SMTP_BUFF_MAX - smtp_checker->buff_ctr);
@@ -409,14 +416,10 @@ smtp_get_line_cb(thread_t *thread)
 	if (r == -1 && (errno == EAGAIN || errno == EINTR)) {
 		thread_add_read(thread->master, smtp_get_line_cb, checker,
 				thread->u.fd, smtp_host->connection_to);
-		fcntl(thread->u.fd, F_SETFL, f);
 		return 0;
 	} else if (r > 0)
 		smtp_checker->buff_ctr += (size_t)r;
 
-	/* restore descriptor flags */
-	fcntl(thread->u.fd, F_SETFL, f);
-
 	/* check if we have a newline, if so, callback */
 	for (x = 0; x < SMTP_BUFF_MAX; x++) {
 		if (smtp_checker->buff[x] == '\n') {
@@ -477,6 +480,7 @@ smtp_get_line(thread_t *thread, int (*callback) (thread_t *))
 	/* schedule the I/O with our helper function  */
 	thread_add_read(thread->master, smtp_get_line_cb, checker,
 		thread->u.fd, smtp_host->connection_to);
+	thread_del_write(thread);
 	return;
 }
 
@@ -492,7 +496,6 @@ smtp_put_line_cb(thread_t *thread)
 	checker_t *checker = THREAD_ARG(thread);
 	smtp_checker_t *smtp_checker = CHECKER_ARG(checker);
 	conn_opts_t *smtp_host = smtp_checker->host_ptr;
-	int f;
 	ssize_t w;
 
 
@@ -503,23 +506,15 @@ smtp_put_line_cb(thread_t *thread)
 		return 0;
 	}
 
-	/* Set descriptor non blocking */
-	f = fcntl(thread->u.fd, F_GETFL, 0);
-	fcntl(thread->u.fd, F_SETFL, f | O_NONBLOCK);
-
 	/* write the data */
 	w = write(thread->u.fd, smtp_checker->buff, smtp_checker->buff_ctr);
 
 	if (w == -1 && (errno == EAGAIN || errno == EINTR)) {
 		thread_add_write(thread->master, smtp_put_line_cb, checker,
 				 thread->u.fd, smtp_host->connection_to);
-		fcntl(thread->u.fd, F_SETFL, f);
 		return 0;
 	}
 
-	/* restore descriptor flags */
-	fcntl(thread->u.fd, F_SETFL, f);
-
 	DBG("SMTP_CHECK %s > %s"
 	    , FMT_SMTP_RS(smtp_host)
 	    , smtp_checker->buff);
@@ -559,6 +554,7 @@ smtp_put_line(thread_t *thread, int (*callback) (thread_t *))
 	/* schedule the I/O with our helper function  */
 	thread_add_write(thread->master, smtp_put_line_cb, checker,
 			 thread->u.fd, smtp_host->connection_to);
+	thread_del_read(thread);
 	return;
 }
 
@@ -573,16 +569,15 @@ smtp_get_status(thread_t *thread)
 	checker_t *checker = THREAD_ARG(thread);
 	smtp_checker_t *smtp_checker = CHECKER_ARG(checker);
 	char *buff = smtp_checker->buff;
+	int status;
+	char *endptr;
 
-	/* First make sure they're all digits */
-	if (isdigit(buff[0]) && isdigit(buff[1]) &&
-	    isdigit(buff[2])) {
-		/* Truncate the string and convert */
-		buff[3] = '\0';
-		return atoi(buff);
-	}
+	status = strtoul(buff, &endptr, 10);
+	if (endptr - buff != 3 ||
+	    (*endptr && *endptr != ' '))
+		return -1;
 
-	return -1;
+	return status;
 }
 
 /*
@@ -619,26 +614,22 @@ smtp_engine_thread(thread_t *thread)
 			if (smtp_get_status(thread) != 220) {
 				smtp_final(thread, 1, "Bad greeting banner from server %s"
 						     , FMT_SMTP_RS(smtp_host));
-
-				return 0;
+			} else {
+				/*
+				 * Schedule to send the HELO, smtp_put_line will
+				 * defer directly to smtp_final on error.
+				 */
+				smtp_checker->state = SMTP_SENT_HELO;
+				snprintf(smtp_checker->buff, SMTP_BUFF_MAX, "HELO %s\r\n",
+					 smtp_checker->helo_name);
+				smtp_put_line(thread, smtp_engine_thread);
 			}
-
-			/*
-			 * Schedule to send the HELO, smtp_put_line will
-			 * defer directly to smtp_final on error.
-			 */
-			smtp_checker->state = SMTP_SENT_HELO;
-			snprintf(smtp_checker->buff, SMTP_BUFF_MAX, "HELO %s\r\n",
-				 smtp_checker->helo_name);
-			smtp_put_line(thread, smtp_engine_thread);
-			return 0;
 			break;
 
 		/* Third step, schedule to read the HELO response */
 		case SMTP_SENT_HELO:
 			smtp_checker->state = SMTP_RECV_HELO;
 			smtp_get_line(thread, smtp_engine_thread);
-			return 0;
 			break;
 
 		/* Fourth step, analyze HELO return, send QUIT */
@@ -647,32 +638,30 @@ smtp_engine_thread(thread_t *thread)
 			if (smtp_get_status(thread) != 250) {
 				smtp_final(thread, 1, "Bad HELO response from server %s"
 						     , FMT_SMTP_RS(smtp_host));
-
-				return 0;
+			} else {
+				smtp_checker->state = SMTP_SENT_QUIT;
+				snprintf(smtp_checker->buff, SMTP_BUFF_MAX, "QUIT\r\n");
+				smtp_put_line(thread, smtp_engine_thread);
 			}
-
-			smtp_checker->state = SMTP_SENT_QUIT;
-			snprintf(smtp_checker->buff, SMTP_BUFF_MAX, "QUIT\r\n");
-			smtp_put_line(thread, smtp_engine_thread);
-			return 0;
 			break;
 
 		/* Fifth step, schedule to receive QUIT confirmation */
 		case SMTP_SENT_QUIT:
 			smtp_checker->state = SMTP_RECV_QUIT;
 			smtp_get_line(thread, smtp_engine_thread);
-			return 0;
 			break;
 
 		/* Sixth step, wrap up success to smtp_final */
 		case SMTP_RECV_QUIT:
 			smtp_final(thread, 0, NULL);
-			return 0;
+			break;
+
+		default:
+			/* We shouldn't be here */
+			smtp_final(thread, 1, "Unknown smtp engine state encountered");
 			break;
 	}
 
-	/* We shouldn't be here */
-	smtp_final(thread, 1, "Unknown smtp engine state encountered");
 	return 0;
 }
 
@@ -693,13 +682,11 @@ smtp_check_thread(thread_t *thread)
 		case connect_error:
 			smtp_final(thread, 1, "Error connecting to server %s"
 					     , FMT_SMTP_RS(smtp_host));
-			return 0;
 			break;
 
 		case connect_timeout:
 			smtp_final(thread, 1, "Connection timeout to server %s"
 					     , FMT_SMTP_RS(smtp_host));
-			return 0;
 			break;
 
 		case connect_success:
@@ -709,13 +696,15 @@ smtp_check_thread(thread_t *thread)
 			/* Enter the engine at SMTP_START */
 			smtp_checker->state = SMTP_START;
 			smtp_engine_thread(thread);
-			return 0;
+			break;
+
+		default:
+			/* we shouldn't be here */
+			smtp_final(thread, 1, "Unknown connection error to server %s"
+					     , FMT_SMTP_RS(smtp_host));
 			break;
 	}
 
-	/* we shouldn't be here */
-	smtp_final(thread, 1, "Unknown connection error to server %s"
-			     , FMT_SMTP_RS(smtp_host));
 	return 0;
 }
 
@@ -737,6 +726,8 @@ smtp_connect_thread(thread_t *thread)
 	conn_opts_t *smtp_host;
 	enum connect_result status;
 	int sd;
+	bool checker_was_up;
+	bool rs_was_alive;
 
 	/* Let's review our data structures.
 	 *
@@ -782,13 +773,17 @@ smtp_connect_thread(thread_t *thread)
 	 * will be reset and we will continue on checking them one by one.
 	 */
 	if ((smtp_checker->host_ptr = list_element(smtp_checker->host, smtp_checker->host_ctr)) == NULL) {
-		if (!checker->is_up) {
+		if (!checker->is_up || !checker->has_run) {
 			log_message(LOG_INFO, "Remote SMTP server %s succeed on service."
 					    , FMT_CHK(checker));
 
-			smtp_alert(checker, NULL, NULL, "UP",
-				   "=> CHECK succeed on service <=");
+			checker_was_up = checker->is_up;
+			rs_was_alive = checker->rs->alive;
 			update_svr_checker_state(UP, checker);
+			if (checker->rs->smtp_alert && !checker_was_up &&
+			    (rs_was_alive != checker->rs->alive || !global_data->no_checker_emails))
+				smtp_alert(SMTP_MSG_RS, checker, NULL,
+					   "=> CHECK succeed on service <=");
 		}
 
 		checker->retry_it = 0;
@@ -801,13 +796,19 @@ smtp_connect_thread(thread_t *thread)
 
 	smtp_host = smtp_checker->host_ptr;
 
-	/* Create the socket, failling here should be an oddity */
-	if ((sd = socket(smtp_host->dst.ss_family, SOCK_STREAM | SOCK_CLOEXEC, IPPROTO_TCP)) == -1) {
+	/* Create the socket, failing here should be an oddity */
+	if ((sd = socket(smtp_host->dst.ss_family, SOCK_STREAM | SOCK_CLOEXEC | SOCK_NONBLOCK, IPPROTO_TCP)) == -1) {
 		log_message(LOG_INFO, "SMTP_CHECK connection failed to create socket. Rescheduling.");
 		thread_add_timer(thread->master, smtp_connect_thread, checker,
 				 checker->delay_loop);
 		return 0;
 	}
+
+#if !HAVE_DECL_SOCK_NONBLOCK
+	if (set_sock_flags(sd, F_SETFL, O_NONBLOCK))
+		log_message(LOG_INFO, "Unable to set NONBLOCK on smtp socket - %s (%d)", strerror(errno), errno);
+#endif
+
 #if !HAVE_DECL_SOCK_CLOEXEC
 	if (set_sock_flags(sd, F_SETFD, FD_CLOEXEC))
 		log_message(LOG_INFO, "Unable to set CLOEXEC on smtp socket - %s (%d)", strerror(errno), errno);
@@ -825,3 +826,15 @@ smtp_connect_thread(thread_t *thread)
 
 	return 0;
 }
+
+
+#ifdef THREAD_DUMP
+void
+register_check_smtp_addresses(void)
+{
+	register_thread_address("smtp_check_thread", smtp_check_thread);
+	register_thread_address("smtp_connect_thread", smtp_connect_thread);
+	register_thread_address("smtp_get_line_cb", smtp_get_line_cb);
+	register_thread_address("smtp_put_line_cb", smtp_put_line_cb);
+}
+#endif
