@@ -17,11 +17,14 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #include "config.h"
 
+#include <unistd.h>
+#include <stdio.h>
+
 #include "check_tcp.h"
 #include "check_api.h"
 #include "memory.h"
@@ -33,7 +36,9 @@
 #include "parser.h"
 #if !HAVE_DECL_SOCK_CLOEXEC
 #include "old_socket.h"
-#include "string.h"
+#endif
+#ifdef THREAD_DUMP
+#include "scheduler.h"
 #endif
 
 static int tcp_connect_thread(thread_t *);
@@ -43,17 +48,16 @@ static void
 free_tcp_check(void *data)
 {
 	FREE(CHECKER_CO(data));
-	FREE(CHECKER_DATA(data));
 	FREE(data);
 }
 
 static void
-dump_tcp_check(void *data)
+dump_tcp_check(FILE *fp, void *data)
 {
 	checker_t *checker = data;
 
-	log_message(LOG_INFO, "   Keepalive method = TCP_CHECK");
-	dump_checker_opts(checker);
+	conf_write(fp, "   Keepalive method = TCP_CHECK");
+	dump_checker_opts(fp, checker);
 }
 
 static bool
@@ -93,6 +97,8 @@ tcp_epilog(thread_t * thread, bool is_success)
 {
 	checker_t *checker;
 	unsigned long delay;
+	bool checker_was_up;
+	bool rs_was_alive;
 
 	checker = THREAD_ARG(thread);
 
@@ -100,24 +106,34 @@ tcp_epilog(thread_t * thread, bool is_success)
 		delay = checker->delay_loop;
 		checker->retry_it = 0;
 
-		if (is_success && !checker->is_up) {
+		if (is_success && (!checker->is_up || !checker->has_run)) {
 			log_message(LOG_INFO, "TCP connection to %s success."
 					, FMT_TCP_RS(checker));
-			smtp_alert(checker, NULL, NULL,
-				   "UP",
-				   "=> TCP CHECK succeed on service <=");
+			checker_was_up = checker->is_up;
+			rs_was_alive = checker->rs->alive;
 			update_svr_checker_state(UP, checker);
-		} else if (!is_success
-			   && checker->is_up) {
+			if (checker->rs->smtp_alert && !checker_was_up &&
+			    (rs_was_alive != checker->rs->alive || !global_data->no_checker_emails))
+				smtp_alert(SMTP_MSG_RS, checker, NULL,
+					   "=> TCP CHECK succeed on service <=");
+		} else if (!is_success && 
+			   (checker->is_up || !checker->has_run)) {
 			if (checker->retry)
 				log_message(LOG_INFO
-				    , "Check on service %s failed after %d retry."
+				    , "Check on service %s failed after %d retries."
 				    , FMT_TCP_RS(checker)
 				    , checker->retry);
-			smtp_alert(checker, NULL, NULL,
-				   "DOWN",
-				   "=> TCP CHECK failed on service <=");
+			else
+				log_message(LOG_INFO
+				    , "Check on service %s failed."
+				    , FMT_TCP_RS(checker));
+			checker_was_up = checker->is_up;
+			rs_was_alive = checker->rs->alive;
 			update_svr_checker_state(DOWN, checker);
+			if (checker->rs->smtp_alert && checker_was_up &&
+			    (rs_was_alive != checker->rs->alive || !global_data->no_checker_emails))
+				smtp_alert(SMTP_MSG_RS, checker, NULL,
+					   "=> TCP CHECK failed on service <=");
 		}
 	} else {
 		delay = checker->delay_before_retry;
@@ -131,10 +147,9 @@ tcp_epilog(thread_t * thread, bool is_success)
 static int
 tcp_check_thread(thread_t * thread)
 {
-	checker_t *checker;
+	checker_t *checker = THREAD_ARG(thread);
 	int status;
 
-	checker = THREAD_ARG(thread);
 	status = tcp_socket_state(thread, tcp_check_thread);
 
 	/* If status = connect_in_progress, next thread is already registered.
@@ -145,7 +160,7 @@ tcp_check_thread(thread_t * thread)
 	case connect_in_progress:
 		break;
 	case connect_success:
-		close(thread->u.fd);
+		thread_close_fd(thread);
 		tcp_epilog(thread, true);
 		break;
 	case connect_timeout:
@@ -182,7 +197,7 @@ tcp_connect_thread(thread_t * thread)
 		return 0;
 	}
 
-	if ((fd = socket(co->dst.ss_family, SOCK_STREAM | SOCK_CLOEXEC, IPPROTO_TCP)) == -1) {
+	if ((fd = socket(co->dst.ss_family, SOCK_STREAM | SOCK_CLOEXEC | SOCK_NONBLOCK, IPPROTO_TCP)) == -1) {
 		log_message(LOG_INFO, "TCP connect fail to create socket. Rescheduling.");
 		thread_add_timer(thread->master, tcp_connect_thread, checker,
 				checker->delay_loop);
@@ -190,6 +205,11 @@ tcp_connect_thread(thread_t * thread)
 		return 0;
 	}
 
+#if !HAVE_DECL_SOCK_NONBLOCK
+	if (set_sock_flags(fd, F_SETFL, O_NONBLOCK))
+		log_message(LOG_INFO, "Unable to set NONBLOCK on tcp_connect socket - %s (%d)", strerror(errno), errno);
+#endif
+
 #if !HAVE_DECL_SOCK_CLOEXEC
 	if (set_sock_flags(fd, F_SETFD, FD_CLOEXEC))
 		log_message(LOG_INFO, "Unable to set CLOEXEC on tcp_connect socket - %s (%d)", strerror(errno), errno);
@@ -208,3 +228,12 @@ tcp_connect_thread(thread_t * thread)
 
 	return 0;
 }
+
+#ifdef THREAD_DUMP
+void
+register_check_tcp_addresses(void)
+{
+	register_thread_address("tcp_check_thread", tcp_check_thread);
+	register_thread_address("tcp_connect_thread", tcp_connect_thread);
+}
+#endif
