@@ -19,15 +19,14 @@
  *              2 of the License, or (at your option) any later version.
  *
  * Copyright (C) 2016 KLab Inc.
+ * Copyright (C) 2016-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #ifndef _CHECK_DNS_CHECK_H
 #define _CHECK_DNS_CHECK_H
 
-#include <stddef.h>
 #include <stdint.h>
-
-#include "scheduler.h"
+#include <sys/types.h>
 
 #define DNS_DEFAULT_RETRY    3
 #define DNS_DEFAULT_TYPE  DNS_TYPE_SOA
@@ -35,22 +34,28 @@
 #define DNS_BUFFER_SIZE    768
 
 #define DNS_QR(flags) ((flags >> 15) & 0x0001)
+/* UNUSED
 #define DNS_OP(flags) ((flags >> 11) & 0x000F)
 #define DNS_AA(flags) ((flags >> 10) & 0x0001)
 #define DNS_TC(flags) ((flags >>  9) & 0x0001)
 #define DNS_RD(flags) ((flags >>  8) & 0x0001)
 #define DNS_RA(flags) ((flags >>  7) & 0x0001)
 #define DNS_Z(flags)  ((flags >>  4) & 0x0007)
+*/
 #define DNS_RC(flags) ((flags >>  0) & 0x000F)
 
+/* UNUSED
 #define DNS_SET_QR(flags, val) (flags |= ((val & 0x0001) << 15))
 #define DNS_SET_OP(flags, val) (flags |= ((val & 0x000F) << 11))
 #define DNS_SET_AA(flags, val) (flags |= ((val & 0x0001) << 10))
 #define DNS_SET_TC(flags, val) (flags |= ((val & 0x0001) <<  9))
+*/
 #define DNS_SET_RD(flags, val) (flags |= ((val & 0x0001) <<  8))
+/* UNUSED
 #define DNS_SET_RA(flags, val) (flags |= ((val & 0x0001) <<  7))
 #define DNS_SET_Z(flags, val)  (flags |= ((val & 0x0007) <<  4))
 #define DNS_SET_RC(flags, val) (flags |= ((val & 0x000F) <<  0))
+*/
 
 #define DNS_TYPE_A       1
 #define DNS_TYPE_NS      2
@@ -88,5 +93,8 @@ typedef struct _dns_check {
 } dns_check_t;
 
 extern void install_dns_check_keyword(void);
+#ifdef THREAD_DUMP
+extern void register_check_dns_addresses(void);
+#endif
 
 #endif
