@@ -6,6 +6,12 @@
 #ifndef KEEPALIVED_IP_VS_H
 #define KEEPALIVED_IP_VS_H
 
+#include "config.h"
+
+/* System includes */
+#include <net/if.h>	/* Force inclusion of net/if.h before linux/if.h */
+#include <sys/types.h>
+#include <netinet/in.h>
 #ifdef HAVE_LINUX_IP_VS_H
 #include <linux/ip_vs.h>
 #else
@@ -18,17 +24,17 @@
 
 #ifdef _WITH_LVS_64BIT_STATS_
 struct ip_vs_stats64 {
-	__u64                   conns;          /* connections scheduled */
-	__u64                   inpkts;         /* incoming packets */
-	__u64                   outpkts;        /* outgoing packets */
-	__u64                   inbytes;        /* incoming bytes */
-	__u64                   outbytes;       /* outgoing bytes */
+	__u64	conns;		/* connections scheduled */
+	__u64	inpkts;		/* incoming packets */
+	__u64	outpkts;	/* outgoing packets */
+	__u64	inbytes;	/* incoming bytes */
+	__u64	outbytes;	/* outgoing bytes */
 
-	__u64			cps;		/* current connection rate */
-	__u64			inpps;		/* current in packet rate */
-	__u64			outpps;		/* current out packet rate */
-	__u64			inbps;		/* current in byte rate */
-	__u64			outbps;		/* current out byte rate */
+	__u64	cps;		/* current connection rate */
+	__u64	inpps;		/* current in packet rate */
+	__u64	outpps;		/* current out packet rate */
+	__u64	inbps;		/* current in byte rate */
+	__u64	outbps;		/* current out byte rate */
 };
 typedef struct ip_vs_stats64 ip_vs_stats_t;
 #else
@@ -100,6 +106,11 @@ struct ip_vs_get_services_app {
 	} user;
 };
 
+/* Make sure we don't have an inconsistent definition */
+#if IP_VS_IFNAME_MAXLEN > IFNAMSIZ
+	#error The code assumes that IP_VS_IFNAME_MAXLEN <= IFNAMSIZ
+#endif
+
 /* The argument to IP_VS_SO_GET_DAEMON */
 struct ip_vs_daemon_kern {
 	/* sync daemon state (master/backup) */
