@@ -17,45 +17,24 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #ifndef _IPVSWRAPPER_H
 #define _IPVSWRAPPER_H
 
-/* system includes */
-#include <unistd.h>
-#include <errno.h>
-#include <sys/socket.h>
-#include <sys/param.h>
-#include <arpa/inet.h>
-#include <asm/types.h>
-#include <stdint.h>
-#include <stdbool.h>
+#include "config.h"
 
-#include <net/if.h>
-//#include <netinet/ip_icmp.h>
-#include <netinet/udp.h>
-#include <netinet/tcp.h>
-#include <sys/wait.h>
-
-#ifdef _WITH_LVS_
-  #include "libipvs.h"
-  #include "ip_vs.h"
 #ifdef _WITH_VRRP_
   #include "vrrp.h"
 #endif
-#endif
 
-/* locale includes */
-#include "scheduler.h"
 #ifdef _WITH_LVS_
 #include "check_data.h"
 #endif
 
 #define IPVS_ERROR	0
 #define IPVS_SUCCESS	1
-#define IPVS_CMD_DELAY	3
 
 #ifdef _WITH_LVS_
 #define IPVS_STARTDAEMON	IP_VS_SO_SET_STARTDAEMON
