@@ -17,18 +17,18 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #ifndef _SMTP_H
 #define _SMTP_H
 
 /* globales includes */
-#include <netdb.h>
+#include <sys/types.h>
+#include <stdbool.h>
 
 /* local includes */
-#include "scheduler.h"
-#include "layer4.h"
+#include "global_data.h"
 #ifdef _WITH_LVS_
 #include "check_data.h"
 #include "check_api.h"
@@ -53,6 +53,19 @@
 #define END	10
 #define ERROR	11
 
+/* SMTP mesage type format */
+typedef enum {
+#ifdef _WITH_LVS_
+	SMTP_MSG_RS,
+	SMTP_MSG_VS,
+	SMTP_MSG_RS_SHUT,
+#endif
+#ifdef _WITH_VRRP_
+	SMTP_MSG_VGROUP,
+	SMTP_MSG_VRRP,
+#endif
+} smtp_msg_t;
+
 /* SMTP thread argument structure */
 #define MAX_HEADERS_LENGTH 256
 #define MAX_BODY_LENGTH    512
@@ -95,7 +108,12 @@ typedef struct _smtp {
 
 #define FMT_SMTP_HOST()	inet_sockaddrtopair(&global_data->smtp_server)
 
-#ifndef _WITH_LVS_
+#ifdef _WITH_LVS_
+typedef struct _smtp_rs {
+	real_server_t *rs;
+	virtual_server_t *vs;
+} smtp_rs;
+#else
 typedef void real_server_t;
 #endif
 #ifndef _WITH_VRRP_
@@ -103,18 +121,14 @@ typedef void vrrp_t;
 typedef void vrrp_sgroup_t;
 #endif
 
+#ifdef _SMTP_ALERT_DEBUG_
+extern bool do_smtp_alert_debug;
+#endif
+
 /* Prototypes defs */
-extern void smtp_alert(
-#ifdef _WITH_LVS_
-			checker_t *,
-#else
-			__attribute__((unused)) void *,
+extern void smtp_alert(smtp_msg_t, void *data, const char *, const char *);
+#ifdef THREAD_DUMP
+extern void register_smtp_addresses(void);
 #endif
-#ifdef _WITH_VRRP_
-			vrrp_t *, vrrp_sgroup_t *,
-#else
-			void *, void *,
-#endif
-			const char *, const char *);
 
 #endif
