@@ -17,19 +17,16 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2016 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #ifndef _VRRP_IPTABLES_H
 #define _VRRP_IPTABLES_H
 
+#include "config.h"
+
 #include <stdbool.h>
 
-#ifdef _HAVE_LIBIPTC_
-#include <libiptc/libxtc.h>
-#endif
-
-#include "vrrp_iptables_calls.h"
 #include "vrrp_ipaddress.h"
 
 struct ipt_handle;
@@ -47,6 +44,7 @@ void iptables_startup(bool);
 void iptables_cleanup(void);
 struct ipt_handle *iptables_open(void);
 int iptables_close(struct ipt_handle *h);
+extern void check_chains_exist_lib(void);
 void handle_iptable_rule_to_vip_lib(ip_address_t *, int, struct ipt_handle *, bool);
 
 #endif
