@@ -17,13 +17,28 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2016-2016 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2016-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 /* See https://git.gnome.org/browse/glib/tree/gio/tests/fdbus-example-server.c
  * and https://developer.gnome.org/gio/stable/GDBusConnection.html#gdbus-server
  * for examples of coding.
  *
+ * Create a general /org/keepalived/Vrrp1/Vrrp DBus
+ * object and a /org/keepalived/Vrrp1/Instance/#interface#/#group# object for
+ * each VRRP instance.
+ * Interface org.keepalived.Vrrp1.Vrrp implements methods PrintData,
+ * PrintStats and signal VrrpStopped.
+ * Interface com.keepalived.Vrrp1.Instance implements method SendGarp
+ * (sends a single Gratuitous ARP from the given Instance),
+ * signal VrrpStatusChange, and properties Name and State (retrievable
+ * through calls to org.freedesktop.DBus.Properties.Get)
+ *
+ * Interface files need to be installed in /usr/share/dbus-1/interfaces/
+ * A policy file, which determines who has access to the service, is
+ * installed in /etc/dbus-1/system.d/. Sources for the policy and interface
+ * files are in keepalived/dbus.
+ *
  * To test the DBus service run a command like: dbus-send --system --dest=org.keepalived.Vrrp1 --print-reply object interface.method type:argument
  * e.g.
  * dbus-send --system --dest=org.keepalived.Vrrp1 --print-reply /org/keepalived/Vrrp1/Vrrp org.keepalived.Vrrp1.Vrrp.PrintData
@@ -38,29 +53,26 @@
 
 #include "config.h"
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
+#include <errno.h>
 #include <pthread.h>
 #include <semaphore.h>
-#include <signal.h>
 #include <gio/gio.h>
 #include <ctype.h>
 #include <fcntl.h>
-#include <unistd.h>
 #include <sys/types.h>
+#include <stdlib.h>
 #include <stdint.h>
 
 #include "vrrp_dbus.h"
 #include "vrrp_data.h"
-#include "vrrp_if.h"
 #include "vrrp_print.h"
 #include "global_data.h"
 #include "main.h"
-#include "memory.h"
 #include "logger.h"
-#include "timer.h"
+#include "utils.h"
+#ifdef THREAD_DUMP
 #include "scheduler.h"
+#endif
 
 typedef enum dbus_action {
 	DBUS_ACTION_NONE,
@@ -160,10 +172,6 @@ state_str(int state)
 		return "Master";
 	case VRRP_STATE_FAULT:
 		return "Fault";
-	case VRRP_STATE_GOTO_MASTER:
-		return "Goto master";
-	case VRRP_STATE_GOTO_FAULT:
-		return "Goto fault";
 	}
 	return "Unknown";
 }
@@ -202,9 +210,9 @@ dbus_object_create_path_vrrp(void)
 {
 	return g_strconcat(DBUS_VRRP_OBJECT_ROOT,
 #if HAVE_DECL_CLONE_NEWNET
-			  (network_namespace) ? "/" : "", (network_namespace) ? network_namespace : "",
+			  global_data->network_namespace ? "/" : "", global_data->network_namespace ? global_data->network_namespace : "",
 #endif
-			  (instance_name) ? "/" : "", (instance_name) ? instance_name : "",
+			  global_data->instance_name ? "/" : "", global_data->instance_name ? global_data->instance_name : "",
 
 			  "/Vrrp", NULL);
 }
@@ -220,9 +228,9 @@ dbus_object_create_path_instance(const gchar *interface, int vrid, sa_family_t f
 
 	object_path = g_strconcat(DBUS_VRRP_OBJECT_ROOT,
 #if HAVE_DECL_CLONE_NEWNET
-				  (network_namespace) ? "/" : "", (network_namespace) ? network_namespace : "",
+				  global_data->network_namespace ? "/" : "", global_data->network_namespace ? global_data->network_namespace : "",
 #endif
-				  (instance_name) ? "/" : "", (instance_name) ? instance_name : "",
+				  global_data->instance_name ? "/" : "", global_data->instance_name ? global_data->instance_name : "",
 
 				  "/Instance/",
 				  standardized_name, "/", vrid_str,
@@ -278,19 +286,22 @@ get_interface_ids(const gchar *object_path, gchar *interface, uint8_t *vrid, uin
 {
 	int path_length = DBUS_VRRP_INSTANCE_PATH_DEFAULT_LENGTH;
 	gchar **dirs;
+	char *endptr;
 
 #if HAVE_DECL_CLONE_NEWNET
-	if(network_namespace)
+	if(global_data->network_namespace)
 		path_length++;
 #endif
-	if(instance_name)
+	if(global_data->instance_name)
 		path_length++;
 
 	/* object_path will have interface, vrid and family as
 	 * the third to last, second to last and last levels */
 	dirs = g_strsplit(object_path, "/", path_length);
 	strcpy(interface, dirs[path_length-3]);
-	*vrid = (uint8_t)atoi(dirs[path_length-2]);
+	*vrid = (uint8_t)strtoul(dirs[path_length-2], &endptr, 10);
+	if (*endptr)
+		log_message(LOG_INFO, "Dbus unexpected characters '%s' at end of number '%s'", endptr, dirs[path_length-2]);
 	*family = !g_strcmp0(dirs[path_length-1], "IPv4") ? AF_INET : !g_strcmp0(dirs[path_length-1], "IPv6") ? AF_INET6 : AF_UNSPEC;
 
 	/* We are finished with all the object_path strings now */
@@ -299,13 +310,13 @@ get_interface_ids(const gchar *object_path, gchar *interface, uint8_t *vrid, uin
 
 /* handles reply to org.freedesktop.DBus.Properties.Get method on any object*/
 static GVariant *
-handle_get_property(__attribute__((unused)) GDBusConnection  *connection,
-		    __attribute__((unused)) const gchar      *sender,
-		    const gchar      *object_path,
-		    const gchar      *interface_name,
-		    const gchar      *property_name,
-		    GError          **error,
-		    __attribute__((unused)) gpointer          user_data)
+handle_get_property(__attribute__((unused)) GDBusConnection *connection,
+		    __attribute__((unused)) const gchar     *sender,
+					    const gchar     *object_path,
+					    const gchar     *interface_name,
+					    const gchar     *property_name,
+					    GError	   **error,
+		    __attribute__((unused)) gpointer	     user_data)
 {
 	GVariant *ret = NULL;
 	dbus_queue_ent_t ent;
@@ -343,16 +354,16 @@ handle_get_property(__attribute__((unused)) GDBusConnection  *connection,
 /* handles method_calls on any object */
 static void
 handle_method_call(__attribute__((unused)) GDBusConnection *connection,
-		   __attribute__((unused)) const gchar           *sender,
-		   const gchar           *object_path,
-		   const gchar           *interface_name,
-		   const gchar           *method_name,
+		   __attribute__((unused)) const gchar	   *sender,
+					   const gchar	   *object_path,
+					   const gchar	   *interface_name,
+					   const gchar	   *method_name,
 #ifndef _WITH_DBUS_CREATE_INSTANCE_
 		   __attribute__((unused))
 #endif
 					   GVariant *parameters,
 		   GDBusMethodInvocation *invocation,
-		   __attribute__((unused)) gpointer               user_data)
+		   __attribute__((unused)) gpointer user_data)
 {
 #ifdef _WITH_DBUS_CREATE_INSTANCE_
 	char *iname;
@@ -511,14 +522,14 @@ dbus_emit_signal(GDBusConnection *connection,
 static void
 on_bus_acquired(GDBusConnection *connection,
 		const gchar     *name,
-		__attribute__((unused)) gpointer         user_data)
+		__attribute__((unused)) gpointer user_data)
 {
 	global_connection = connection;
 	gchar *path;
 	element e;
 	GError *local_error = NULL;
 
-	log_message(LOG_INFO, "Acquired DBus bus %s\n", name);
+	log_message(LOG_INFO, "Acquired DBus bus %s", name);
 
 	/* register VRRP object */
 	path = dbus_object_create_path_vrrp();
@@ -558,9 +569,9 @@ on_bus_acquired(GDBusConnection *connection,
 static void
 on_name_acquired(__attribute__((unused)) GDBusConnection *connection,
 		 const gchar     *name,
-		 __attribute__((unused)) gpointer         user_data)
+		 __attribute__((unused)) gpointer user_data)
 {
-	log_message(LOG_INFO, "Acquired the name %s on the session bus\n", name);
+	log_message(LOG_INFO, "Acquired the name %s on the session bus", name);
 }
 
 /* run if bus name or connection are lost */
@@ -569,7 +580,7 @@ on_name_lost(GDBusConnection *connection,
 	     const gchar     *name,
 	     __attribute__((unused)) gpointer user_data)
 {
-	log_message(LOG_INFO, "Lost the name %s on the session bus\n", name);
+	log_message(LOG_INFO, "Lost the name %s on the session bus", name);
 	global_connection = connection;
 	g_hash_table_foreach_remove(objects, unregister_object, NULL);
 	objects = NULL;
@@ -583,14 +594,14 @@ read_file(gchar* filepath)
 	size_t length;
 	gchar *ret = NULL;
 
-	f = fopen(filepath, "rb");
+	f = fopen(filepath, "r");
 	if (f) {
 		fseek(f, 0, SEEK_END);
 		length = (size_t)ftell(f);
 		fseek(f, 0, SEEK_SET);
 
 		/* We can't use MALLOC since it isn't thread safe */
-		ret = malloc(length + 1);
+		ret = MALLOC(length + 1);
 		if (ret) {
 			if (fread(ret, length, 1, f) != 1) {
 				log_message(LOG_INFO, "Failed to read all of %s", filepath);
@@ -630,7 +641,7 @@ dbus_main(__attribute__ ((unused)) void *unused)
 	if (!introspection_xml)
 		return NULL;
 	vrrp_introspection_data = g_dbus_node_info_new_for_xml(introspection_xml, &error);
-	free(introspection_xml);
+	FREE(introspection_xml);
 	if (error != NULL) {
 		log_message(LOG_INFO, "Parsing DBus interface %s from file %s failed: %s",
 			    DBUS_VRRP_INTERFACE, DBUS_VRRP_INTERFACE_FILE_PATH, error->message);
@@ -642,7 +653,7 @@ dbus_main(__attribute__ ((unused)) void *unused)
 	if (!introspection_xml)
 		return NULL;
 	vrrp_instance_introspection_data = g_dbus_node_info_new_for_xml(introspection_xml, &error);
-	free(introspection_xml);
+	FREE(introspection_xml);
 	if (error != NULL) {
 		log_message(LOG_INFO, "Parsing DBus interface %s from file %s failed: %s",
 			    DBUS_VRRP_INSTANCE_INTERFACE, DBUS_VRRP_INSTANCE_INTERFACE_FILE_PATH, error->message);
@@ -870,37 +881,21 @@ dbus_start(void)
 	pthread_t dbus_thread;
 	sigset_t sigset, cursigset;
 
-#ifdef HAVE_PIPE2
-	if (pipe2(dbus_in_pipe, O_CLOEXEC)) {
+	if (open_pipe(dbus_in_pipe)) {
 		log_message(LOG_INFO, "Unable to create inbound dbus pipe - disabling DBus");
 		return false;
 	}
-	if (pipe2(dbus_out_pipe, O_CLOEXEC)) {
+	if (open_pipe(dbus_out_pipe)) {
 		log_message(LOG_INFO, "Unable to create outbound dbus pipe - disabling DBus");
 		close(dbus_in_pipe[0]);
 		close(dbus_in_pipe[1]);
 		return false;
 	}
-#else
-	if (pipe(dbus_in_pipe)) {
-		log_message(LOG_INFO, "Unable to create inbound dbus pipe - disabling DBus");
-		return false;
-	}
-	if (pipe(dbus_out_pipe)) {
-		log_message(LOG_INFO, "Unable to create outbound dbus pipe - disabling DBus");
-		close(dbus_in_pipe[0]);
-		close(dbus_in_pipe[1]);
-		return false;
-	}
-	fcntl (dbus_in_pipe[0], F_SETFD, FD_CLOEXEC | fcntl(dbus_in_pipe[0], F_GETFD));
-	fcntl (dbus_in_pipe[1], F_SETFD, FD_CLOEXEC | fcntl(dbus_in_pipe[1], F_GETFD));
-	fcntl (dbus_out_pipe[0], F_SETFD, FD_CLOEXEC | fcntl(dbus_out_pipe[0], F_GETFD));
-	fcntl (dbus_out_pipe[1], F_SETFD, FD_CLOEXEC | fcntl(dbus_out_pipe[1], F_GETFD));
-#endif
 
-	/* We don't want the main thread to block when using the pipes */
-	fcntl(dbus_in_pipe[0], F_SETFL, O_NONBLOCK | fcntl(dbus_in_pipe[0], F_GETFL));
-	fcntl(dbus_out_pipe[1], F_SETFL, O_NONBLOCK | fcntl(dbus_out_pipe[1], F_GETFL));
+	/* We don't want the main thread to block when using the pipes,
+	 * but the other side of the pipes should block. */
+	fcntl(dbus_in_pipe[1], F_SETFL, fcntl(dbus_in_pipe[1], F_GETFL) & ~O_NONBLOCK);
+	fcntl(dbus_out_pipe[0], F_SETFL, fcntl(dbus_out_pipe[0], F_GETFL) & ~O_NONBLOCK);
 
 	thread_add_read(master, handle_dbus_msg, NULL, dbus_in_pipe[0], TIMER_NEVER);
 
@@ -908,7 +903,7 @@ dbus_start(void)
 	sem_init(&thread_end, 0, 0);
 
 	/* Block signals (all) we don't want the new thread to process */
-	sigemptyset(&sigset);
+	sigfillset(&sigset);
 	pthread_sigmask(SIG_SETMASK, &sigset, &cursigset);
 
 	/* Now create the dbus thread */
@@ -953,3 +948,11 @@ dbus_stop(void)
 		sem_destroy(&thread_end);
 	}
 }
+
+#ifdef THREAD_DUMP
+void
+register_vrrp_dbus_addresses(void)
+{
+	register_thread_address("handle_dbus_msg", handle_dbus_msg);
+}
+#endif
