@@ -18,6 +18,7 @@
  *              2 of the License, or (at your option) any later version.
  *
  * Copyright (C) 2015 Chris Riley, <kernelchris@gmail.com>
+ * Copyright (C) 2016-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #include "config.h"
@@ -28,6 +29,8 @@
 #include <stdlib.h>
 #include <math.h>
 #include <arpa/inet.h>
+#include <stdint.h>
+#include <ctype.h>
 
 #include "logger.h"
 #include "vrrp_ip_rule_route_parser.h"
@@ -35,6 +38,7 @@
 #if HAVE_DECL_RTA_ENCAP
 #include "vrrp_iproute.h"
 #endif
+#include "parser.h"
 
 bool
 get_realms(uint32_t *realms, char *str)
@@ -72,76 +76,56 @@ err:
 bool
 get_u8(uint8_t *val, const char *str, uint8_t max, const char* errmsg)
 {
-	char *end;
-	unsigned long t_val;
+	unsigned t_val;
 
-	t_val = strtoul(str, &end, 0);
-	if (*end == '\0' && t_val <= max) {
+	if (read_unsigned(str, &t_val, 0, max, false)) {
 		*val = (uint8_t)t_val;
 		return false;
 	}
 
-	log_message(LOG_INFO, errmsg, str);
+	report_config_error(CONFIG_GENERAL_ERROR, errmsg, str);
 	return true;
 }
 
 bool
 get_u16(uint16_t *val, const char *str, uint16_t max, const char* errmsg)
 {
-	char *end;
-	unsigned long t_val;
+	unsigned t_val;
 
-	/* strtoul can do "nasty" things with -ve unsigneds */
-	if (str[0] == '-')
-		return true;
-
-	t_val = strtoul(str, &end, 0);
-	if (*end == '\0' && t_val <= max) {
+	if (read_unsigned(str, &t_val, 0, max, false)) {
 		*val = (uint16_t)t_val;
 		return false;
 	}
 
-	log_message(LOG_INFO, errmsg, str);
+	report_config_error(CONFIG_GENERAL_ERROR, errmsg, str);
 	return true;
 }
 
 bool
 get_u32(uint32_t *val, const char *str, uint32_t max, const char* errmsg)
 {
-	char *end;
-	unsigned long t_val;
+	unsigned t_val;
 
-	/* strtoul can do "nasty" things with -ve unsigneds */
-	if (str[0] == '-')
-		return true;
-
-	t_val = strtoul(str, &end, 0);
-	if (*end == '\0' && t_val <= max) {
+	if (read_unsigned(str, &t_val, 0, max, false)) {
 		*val = (uint32_t)t_val;
 		return false;
 	}
 
-	log_message(LOG_INFO, errmsg, str);
+	report_config_error(CONFIG_GENERAL_ERROR, errmsg, str);
 	return true;
 }
 
 bool
 get_u64(uint64_t *val, const char *str, uint64_t max, const char* errmsg)
 {
-	char *end;
-	unsigned long long t_val;
+	uint64_t t_val;
 
-	/* strtoull can do "nasty" things with -ve unsigneds */
-	if (str[0] == '-')
-		return true;
-
-	t_val = strtoull(str, &end, 0);
-	if (*end == '\0' && t_val <= max) {
-		*val = t_val;
+	if (read_unsigned64(str, &t_val, 0, max, false)) {
+		*val = (uint64_t)t_val;
 		return false;
 	}
 
-	log_message(LOG_INFO, errmsg, str);
+	report_config_error(CONFIG_GENERAL_ERROR, errmsg, str);
 	return true;
 }
 
@@ -151,6 +135,7 @@ get_time_rtt(uint32_t *val, const char *str, bool *raw)
 	double t;
 	unsigned long res;
 	char *end;
+	size_t offset;
 
 	errno = 0;
 	if (strchr(str, '.') ||
@@ -167,11 +152,14 @@ get_time_rtt(uint32_t *val, const char *str, bool *raw)
 		if (t == HUGE_VAL && errno == ERANGE)
 			return true;
 
-		if (t >=UINT32_MAX)
+		if (t >= UINT32_MAX)
 			return true;
 	} else {
+		/* Skip whitespace */
+		offset = strspn(str, WHITE_SPACE);
+
 		/* strtoul does "nasty" things with negative numbers */
-		if (str[0] == '-')
+		if (str[offset] == '-')
 			return true;
 
 		res = strtoul(str, &end, 0);
@@ -184,7 +172,7 @@ get_time_rtt(uint32_t *val, const char *str, bool *raw)
 		if (res == ULONG_MAX && errno == ERANGE)
 			return true;
 
-		if (res >= UINT32_MAX)
+		if (res > UINT32_MAX)
 			return true;
 
 		t = (double)res;
@@ -196,7 +184,7 @@ get_time_rtt(uint32_t *val, const char *str, bool *raw)
 		    !strcasecmp(end, "sec") ||
 		    !strcasecmp(end, "secs")) {
 			if (t >= UINT32_MAX / 1000)
-				return -1;
+				return false;
 			t *= 1000;
 		}
 		else if (strcasecmp(end, "ms") &&
@@ -210,7 +198,7 @@ get_time_rtt(uint32_t *val, const char *str, bool *raw)
 	*val = (uint32_t)t;
 	if (*val < t)
 		(*val)++;
-	
+
 	return false;
 }
 
@@ -224,11 +212,17 @@ get_addr64(uint64_t *ap, const char *cp)
 		uint64_t v64;
 	} val;
 
+	/* Skip leading whitespace */
+	cp += strspn(cp, WHITE_SPACE);
+
 	val.v64 = 0;
 	for (i = 0; i < 4; i++) {
 		unsigned long n;
 		char *endp;
 
+		if (!isxdigit(*cp))
+			return true;	/* Not a hex digit */
+
 		n = strtoul(cp, &endp, 16);
 		if (n > 0xffff)
 			return true;	/* bogus network value */
@@ -254,7 +248,7 @@ get_addr64(uint64_t *ap, const char *cp)
 	return false;
 }
 
-#if HAVE_DECL_RTA_ENCAP
+#if HAVE_DECL_LWTUNNEL_ENCAP_MPLS
 bool
 parse_mpls_address(const char *str, encap_mpls_t *mpls)
 {
@@ -264,10 +258,16 @@ parse_mpls_address(const char *str, encap_mpls_t *mpls)
 
 	mpls->num_labels = 0;
 
+	/* Skip leading whitespace */
+	str += strspn(str, WHITE_SPACE);
+
 	for (count = 0; count < MAX_MPLS_LABELS; count++) {
 		if (str[0] == '-')
 			return true;
 
+		if (strspn(str, WHITE_SPACE))	/* No embedded whitespace */
+			return true;
+
 		label = strtoul(str, &endp, 0);
 
 		if (endp == str) /* no digits */
@@ -292,6 +292,7 @@ parse_mpls_address(const char *str, encap_mpls_t *mpls)
 
 		str = endp + 1;
 	}
+
 	/* The address was too long */
 	return true;
 }
