@@ -18,27 +18,47 @@
  *              2 of the License, or (at your option) any later version.
  *
  * Copyright (C) 2015 Chris Riley, <kernelchris@gmail.com>
+ * Copyright (C) 2016-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #include "config.h"
 
 /* global includes */
+#include <errno.h>
+#ifdef NETLINK_H_NEEDS_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
 #include <linux/fib_rules.h>
 #include <inttypes.h>
+#include <stdlib.h>
+#include <stdint.h>
+#include <stdio.h>
+#if HAVE_DECL_FRA_IP_PROTO
+#include <netdb.h>
+#include <inttypes.h>
+#endif
+#include <ctype.h>
 
 /* local include */
-#include "vrrp_ipaddress.h"
 #include "vrrp_iproute.h"
 #include "vrrp_iprule.h"
-#include "vrrp_netlink.h"
-#include "vrrp_if.h"
+#include "keepalived_netlink.h"
 #include "vrrp_data.h"
 #include "logger.h"
 #include "memory.h"
 #include "utils.h"
 #include "rttables.h"
 #include "vrrp_ip_rule_route_parser.h"
+#include "parser.h"
+
+/* Since we will be adding and deleting rules in potentially random
+ * orders due to master/backup transitions, we therefore need to
+ * pre-allocate priorities to ensure the rules are added in a consistent
+ * sequence. Really the configuration should specify a priority for each
+ * rule to ensure they are configured in the order the user wants. */
+#define RULE_START_PRIORITY 16384
+static unsigned next_rule_priority_ipv4 = RULE_START_PRIORITY;
+static unsigned next_rule_priority_ipv6 = RULE_START_PRIORITY;
 
 /* Utility functions */
 static inline bool
@@ -62,21 +82,69 @@ rule_is_equal(const ip_rule_t *x, const ip_rule_t *y)
 #if HAVE_DECL_FRA_TUN_ID
 	    x->tunnel_id != y->tunnel_id ||
 #endif
-	    !(x->iif) != !(y->iif) ||
-	    !(x->oif) != !(y->oif) ||
+#if HAVE_DECL_FRA_UID_RANGE
+	    x->uid_range.start != y->uid_range.start ||
+	    x->uid_range.end != y->uid_range.end ||
+#endif
+#if HAVE_DECL_FRA_L3MDEV
+	    x->l3mdev != y->l3mdev ||
+#endif
+	    x->iif != y->iif ||
+#if HAVE_DECL_FRA_OIFNAME
+	    x->oif != y->oif ||
+#endif
+#if HAVE_DECL_FRA_PROTOCOL
+	    x->protocol != y->protocol ||
+#endif
+#if HAVE_DECL_FRA_IP_PROTO
+	    x->ip_proto != y->ip_proto ||
+#endif
+#if HAVE_DECL_FRA_SPORT_RANGE
+	    x->src_port.start != y->src_port.start ||
+	    x->src_port.end != y->src_port.end ||
+#endif
+#if HAVE_DECL_FRA_DPORT_RANGE
+	    x->dst_port.start != y->dst_port.start ||
+	    x->dst_port.end != y->dst_port.end ||
+#endif
 	    x->goto_target != y->goto_target ||
 	    x->table != y->table ||
 	    x->action != y->action)
 		return false;
 
-	if (x->iif && x->iif->ifindex != y->iif->ifindex)
-		return false;
-	if (x->oif && x->oif->ifindex != y->oif->ifindex)
-		return false;
-
 	return true;
 }
 
+#if HAVE_DECL_FRA_IP_PROTO
+static int
+inet_proto_a2n(const char *buf)
+{
+	struct protoent *pe;
+	unsigned long proto_num;
+	char *endptr;
+
+	/* Skip white space */
+	buf += strspn(buf, WHITE_SPACE);
+
+	if (!*buf || *buf == '-')
+		return -1;
+
+	proto_num = strtoul(buf, &endptr, 10);
+	if (proto_num > INT8_MAX)
+		return -1;
+	if (!*endptr)
+		return proto_num;
+
+	pe = getprotobyname(buf);
+	endprotoent();
+
+	if (pe)
+		return pe->p_proto;
+
+	return -1;
+}
+#endif
+
 /* Add/Delete IP rule to/from a specific IP/network */
 static int
 netlink_rule(ip_rule_t *iprule, int cmd)
@@ -104,15 +172,14 @@ netlink_rule(ip_rule_t *iprule, int cmd)
 	}
 	req.frh.table = RT_TABLE_UNSPEC;
 	req.frh.flags = 0;
+	req.frh.tos = iprule->tos;	// Hex value - 0xnn <= 255, or name from rt_dsfield
+	req.frh.family = iprule->family;
 
-	if (iprule->from_addr)
-		req.frh.family = IP_FAMILY(iprule->from_addr);
-	else if (iprule->to_addr)
-		req.frh.family = IP_FAMILY(iprule->to_addr);
-	else
-		req.frh.family = AF_INET;
-
-	if (iprule->action == FR_ACT_TO_TBL) {
+	if (iprule->action == FR_ACT_TO_TBL
+#if HAVE_DECL_FRA_L3MDEV
+	    && !iprule->l3mdev
+#endif
+					   ) {
 		if (iprule->table < 256)	// "Table" or "lookup"
 			req.frh.table = iprule->table ? iprule->table & 0xff : RT_TABLE_MAIN;
 		else {
@@ -137,9 +204,6 @@ netlink_rule(ip_rule_t *iprule, int cmd)
 	if (iprule->mask & IPRULE_BIT_PRIORITY)	// "priority/order/preference"
 		addattr32(&req.n, sizeof(req), FRA_PRIORITY, iprule->priority);
 
-	if (iprule->mask & IPRULE_BIT_DSFIELD)	// "tos/dsfield"
-		req.frh.tos = iprule->tos;	// Hex value - 0xnn <= 255, or name from rt_dsfield
-
 	if (iprule->mask & IPRULE_BIT_FWMARK)	// "fwmark"
 		addattr32(&req.n, sizeof(req), FRA_FWMARK, iprule->fwmark);
 
@@ -150,7 +214,7 @@ netlink_rule(ip_rule_t *iprule, int cmd)
 		addattr32(&req.n, sizeof(req), FRA_FLOW, iprule->realms);
 
 #if HAVE_DECL_FRA_SUPPRESS_PREFIXLEN
-	if (iprule->mask & IPRULE_BIT_SUP_PREFIXLEN)	// "suppress_prefixlength" - only valid if table !=0
+	if (iprule->suppress_prefix_len != -1)	// "suppress_prefixlength" - only valid if table != 0
 		addattr32(&req.n, sizeof(req), FRA_SUPPRESS_PREFIXLEN, iprule->suppress_prefix_len);
 #endif
 
@@ -171,6 +235,37 @@ netlink_rule(ip_rule_t *iprule, int cmd)
 	if (iprule->tunnel_id)
 		addattr64(&req.n, sizeof(req), FRA_TUN_ID, htobe64(iprule->tunnel_id));
 #endif
+
+#if HAVE_DECL_FRA_UID_RANGE
+	if (iprule->mask & IPRULE_BIT_UID_RANGE)
+		addattr_l(&req.n, sizeof(req), FRA_UID_RANGE, &iprule->uid_range, sizeof(iprule->uid_range));
+#endif
+
+#if HAVE_DECL_FRA_L3MDEV
+	if (iprule->l3mdev)
+		addattr8(&req.n, sizeof(req), FRA_L3MDEV, 1);
+#endif
+
+#if HAVE_DECL_FRA_PROTOCOL
+	if (iprule->mask & IPRULE_BIT_PROTOCOL)
+		addattr8(&req.n, sizeof(req), FRA_PROTOCOL, iprule->protocol);
+#endif
+
+#if HAVE_DECL_FRA_IP_PROTO
+	if (iprule->mask & IPRULE_BIT_IP_PROTO)
+		addattr8(&req.n, sizeof(req), FRA_IP_PROTO, iprule->ip_proto);
+#endif
+
+#if HAVE_DECL_FRA_SPORT_RANGE
+	if (iprule->mask & IPRULE_BIT_SPORT_RANGE)
+		addattr_l(&req.n, sizeof(req), FRA_SPORT_RANGE, &iprule->src_port, sizeof(iprule->src_port));
+#endif
+
+#if HAVE_DECL_FRA_DPORT_RANGE
+	if (iprule->mask & IPRULE_BIT_DPORT_RANGE)
+		addattr_l(&req.n, sizeof(req), FRA_DPORT_RANGE, &iprule->dst_port, sizeof(iprule->dst_port));
+#endif
+
 	if (iprule->action == FR_ACT_GOTO) {	// "goto"
 		addattr32(&req.n, sizeof(req), FRA_GOTO, iprule->goto_target);
 		req.frh.action = FR_ACT_GOTO;
@@ -184,6 +279,17 @@ netlink_rule(ip_rule_t *iprule, int cmd)
 	return status;
 }
 
+void
+reinstate_static_rule(ip_rule_t *rule)
+{
+	char buf[256];
+
+	rule->set = (netlink_rule(rule, IPRULE_ADD) > 0);
+
+	format_iprule(rule, buf, sizeof(buf));
+	log_message(LOG_INFO, "Restoring deleted static rule %s", buf);
+}
+
 void
 netlink_rulelist(list rule_list, int cmd, bool force)
 {
@@ -198,7 +304,7 @@ netlink_rulelist(list rule_list, int cmd, bool force)
 	 * rule might not exist. That's not an error, so indicate not
 	 * to report such a situation */
 	if (force && cmd == IPRULE_DEL)
-	         netlink_error_ignore = ENOENT;
+		netlink_error_ignore = ENOENT;
 
 	for (e = LIST_HEAD(rule_list); e; ELEMENT_NEXT(e)) {
 		iprule = ELEMENT_DATA(e);
@@ -232,8 +338,11 @@ format_iprule(ip_rule_t *rule, char *buf, size_t buf_len)
 	char *op = buf;
 	char *buf_end = buf + buf_len;
 
+	if (!rule->to_addr && !rule->from_addr && rule->family == AF_INET6)
+		op += snprintf(op, (size_t)(buf_end - op), "inet6 ");
+
 	if (rule->invert)
-		op += snprintf(op, (size_t)(buf_end - op), " not");
+		op += snprintf(op, (size_t)(buf_end - op), "not ");
 
 	if (rule->from_addr) {
 		op += snprintf(op, (size_t)(buf_end - op), "from %s", ipaddresstos(NULL, rule->from_addr));
@@ -254,8 +363,7 @@ format_iprule(ip_rule_t *rule, char *buf, size_t buf_len)
 	if (rule->mask & IPRULE_BIT_PRIORITY)
 		op += snprintf(op, (size_t)(buf_end - op), " priority %u", rule->priority);
 
-	if (rule->mask & IPRULE_BIT_DSFIELD)
-		op += snprintf(op, (size_t)(buf_end - op), " tos 0x%x", rule->tos);
+	op += snprintf(op, (size_t)(buf_end - op), " tos 0x%x", rule->tos);
 
 	if (rule->mask & (IPRULE_BIT_FWMARK | IPRULE_BIT_FWMASK)) {
 		op += snprintf(op, (size_t)(buf_end - op), " fwmark 0x%x", rule->fwmark);
@@ -277,7 +385,7 @@ format_iprule(ip_rule_t *rule, char *buf, size_t buf_len)
 #endif
 
 #if HAVE_DECL_FRA_SUPPRESS_PREFIXLEN
-	if (rule->mask & IPRULE_BIT_SUP_PREFIXLEN)
+	if (rule->suppress_prefix_len != -1)
 		op += snprintf(op, (size_t)(buf_end - op), " suppress_prefixlen %u", rule->suppress_prefix_len);
 #endif
 
@@ -291,6 +399,36 @@ format_iprule(ip_rule_t *rule, char *buf, size_t buf_len)
 		op += snprintf(op, (size_t)(buf_end - op), " tunnel-id %" PRIu64, rule->tunnel_id);
 #endif
 
+#if HAVE_DECL_FRA_UID_RANGE
+	if (rule->mask & IPRULE_BIT_UID_RANGE)
+		op += snprintf(op, (size_t)(buf_end - op), " uidrange %" PRIu32 "-%" PRIu32, rule->uid_range.start, rule->uid_range.end);
+#endif
+
+#if HAVE_DECL_FRA_L3MDEV
+	if (rule->l3mdev)
+		op += snprintf(op, (size_t)(buf_end - op), " l3mdev");
+#endif
+
+#if HAVE_DECL_FRA_PROTOCOL
+	if (rule->mask & IPRULE_BIT_PROTOCOL)
+		op += snprintf(op, (size_t)(buf_end - op), " protocol %u", rule->protocol);
+#endif
+
+#if HAVE_DECL_FRA_IP_PROTO
+	if (rule->mask & IPRULE_BIT_IP_PROTO)
+		op += snprintf(op, (size_t)(buf_end - op), " ipproto %u", rule->ip_proto);
+#endif
+
+#if HAVE_DECL_FRA_SPORT_RANGE
+	if (rule->mask & IPRULE_BIT_SPORT_RANGE)
+		op += snprintf(op, (size_t)(buf_end - op), " sport %hu-%hu", rule->src_port.start, rule->src_port.end);
+#endif
+
+#if HAVE_DECL_FRA_DPORT_RANGE
+	if (rule->mask & IPRULE_BIT_DPORT_RANGE)
+		op += snprintf(op, (size_t)(buf_end - op), " dport %hu-%hu", rule->dst_port.start, rule->dst_port.end);
+#endif
+
 	if (rule->realms)
 		op += snprintf(op, (size_t)(buf_end - op), " realms %d/%d", rule->realms >> 16, rule->realms & 0xffff);
 
@@ -302,28 +440,33 @@ format_iprule(ip_rule_t *rule, char *buf, size_t buf_len)
 		op += snprintf(op, (size_t)(buf_end - op), " nop");
 	else
 		op += snprintf(op, (size_t)(buf_end - op), " type %s", get_rttables_rtntype(rule->action));
+	if (rule->dont_track)
+		op += snprintf(op, (size_t)(buf_end - op), " no_track");
+	if (rule->track_group)
+		op += snprintf(op, (size_t)(buf_end - op), " track_group %s", rule->track_group->gname);
 }
 
 void
-dump_iprule(void *rule_data)
+dump_iprule(FILE *fp, void *rule_data)
 {
 	ip_rule_t *rule = rule_data;
 	char *buf = MALLOC(RULE_BUF_SIZE);
 
 	format_iprule(rule, buf, RULE_BUF_SIZE);
 
-	log_message(LOG_INFO, "     %s", buf);
+	conf_write(fp, "     %s", buf);
 
 	FREE(buf);
 }
 
 void
-alloc_rule(list rule_list, vector_t *strvec)
+alloc_rule(list rule_list, vector_t *strvec, __attribute__((unused)) bool allow_track_group)
 {
 	ip_rule_t *new;
 	char *str;
 	unsigned int i = 0;
 	unsigned long val, val1;
+	unsigned val_unsigned;
 	uint32_t uval32;
 	uint8_t uval8;
 	int family = AF_UNSPEC;
@@ -338,56 +481,78 @@ alloc_rule(list rule_list, vector_t *strvec)
 	}
 
 	new->action = FR_ACT_UNSPEC;
+#if HAVE_DECL_FRA_SUPPRESS_PREFIXLEN
+	new->suppress_prefix_len = -1;
+#endif
 
 	/* FMT parse */
 	while (i < vector_size(strvec)) {
 		str = strvec_slot(strvec, i);
 
-		if (!strcmp(str, "from")) {
+		/* Check if inet4/6 specified */
+		if (!strcmp(str, "inet6")) {
+			if (family == AF_UNSPEC)
+				family = AF_INET6;
+			else if (family != AF_INET6) {
+				report_config_error(CONFIG_GENERAL_ERROR, "inet6 specified for IPv4 rule");
+				goto err;
+			}
+			i++;
+		}
+		else if (!strcmp(str, "inet")) {
+			if (family == AF_UNSPEC)
+				family = AF_INET;
+			else if (family != AF_INET) {
+				report_config_error(CONFIG_GENERAL_ERROR, "inet specified for IPv6 rule");
+				goto err;
+			}
+			i++;
+		}
+		else if (!strcmp(str, "from")) {
 			if (new->from_addr)
 				FREE(new->from_addr);
-			new->from_addr = parse_ipaddress(NULL, strvec_slot(strvec, ++i), false);
+			new->from_addr = parse_route(strvec_slot(strvec, ++i));
 			if (!new->from_addr) {
-				log_message(LOG_INFO, "Invalid rule from address %s", FMT_STR_VSLOT(strvec, i));
+				report_config_error(CONFIG_GENERAL_ERROR, "Invalid rule from address %s", FMT_STR_VSLOT(strvec, i));
 				goto err;
 			}
 			if (family == AF_UNSPEC)
 				family = new->from_addr->ifa.ifa_family;
 			else if (new->from_addr->ifa.ifa_family != family)
 			{
-				log_message(LOG_INFO, "rule specification has mixed IPv4 and IPv6");
+				report_config_error(CONFIG_GENERAL_ERROR, "rule specification has mixed IPv4 and IPv6");
 				goto err;
 			}
 		}
 		else if (!strcmp(str, "to")) {
 			if (new->to_addr)
 				FREE(new->to_addr);
-			new->to_addr = parse_ipaddress(NULL, strvec_slot(strvec, ++i), false);
+			new->to_addr = parse_route(strvec_slot(strvec, ++i));
 			if (!new->to_addr) {
-				log_message(LOG_INFO, "Invalid rule to address %s", FMT_STR_VSLOT(strvec, i));
+				report_config_error(CONFIG_GENERAL_ERROR, "Invalid rule to address %s", FMT_STR_VSLOT(strvec, i));
 				goto err;
 			}
 			if (family == AF_UNSPEC)
 				family = new->to_addr->ifa.ifa_family;
 			else if (new->to_addr->ifa.ifa_family != family)
 			{
-				log_message(LOG_INFO, "rule specification has mixed IPv4 and IPv6");
+				report_config_error(CONFIG_GENERAL_ERROR, "rule specification has mixed IPv4 and IPv6");
 				goto err;
 			}
 		}
 		else if (!strcmp(str, "table") ||
 			 !strcmp(str, "lookup")) {
 			if (!find_rttables_table(strvec_slot(strvec, ++i), &uval32)) {
-				log_message(LOG_INFO, "Routing table %s not found for rule", FMT_STR_VSLOT(strvec, i));
+				report_config_error(CONFIG_GENERAL_ERROR, "Routing table %s not found for rule", FMT_STR_VSLOT(strvec, i));
 				goto err;
 			}
 			if (uval32 == 0) {
-				log_message(LOG_INFO, "Table 0 is not valid");
+				report_config_error(CONFIG_GENERAL_ERROR, "Table 0 is not valid");
 				goto err;
 			}
 			new->table = uval32;
 			if (new->action != FR_ACT_UNSPEC) {
-				log_message(LOG_INFO, "Cannot specify more than one of table/nop/goto/blackhole/prohibit/unreachable for rule");
+				report_config_error(CONFIG_GENERAL_ERROR, "Cannot specify more than one of table/nop/goto/blackhole/prohibit/unreachable for rule");
 				goto err;
 			}
 			new->action = FR_ACT_TO_TBL;
@@ -397,27 +562,25 @@ alloc_rule(list rule_list, vector_t *strvec)
 		else if (!strcmp(str, "preference") ||
 			 !strcmp(str, "order") ||
 			 !strcmp(str, "priority")) {
-			str = strvec_slot(strvec, ++i);
-			val = strtoul(str, &end, 0);
-			if (*end || val > UINT32_MAX) {
-				log_message(LOG_INFO, "Invalid rule preference %s specified", str);
+			if (!read_unsigned_base_strvec(strvec, ++i, 0, &val_unsigned, 0, UINT32_MAX, false)) {
+				report_config_error(CONFIG_GENERAL_ERROR, "Invalid rule preference %s specified", str);
 				goto err;
 			}
 
-			new->priority = (uint32_t)val;
+			new->priority = (uint32_t)val_unsigned;
 			new->mask |= IPRULE_BIT_PRIORITY;
 		}
 		else if (!strcmp(str, "tos") || !strcmp(str, "dsfield")) {
 			if (!find_rttables_dsfield(strvec_slot(strvec, ++i), &uval8)) {
-				log_message(LOG_INFO, "TOS value %s is invalid", FMT_STR_VSLOT(strvec, i));
+				report_config_error(CONFIG_GENERAL_ERROR, "TOS value %s is invalid", FMT_STR_VSLOT(strvec, i));
 				goto err;
 			}
 
 			new->tos = uval8;
-			new->mask |= IPRULE_BIT_DSFIELD;
 		}
 		else if (!strcmp(str, "fwmark")) {
 			str = strvec_slot(strvec, ++i);
+			str += strspn(str, WHITE_SPACE);
 			if (str[0] == '-')
 				goto fwmark_err;
 			val = strtoul(str, &end, 0);
@@ -425,10 +588,10 @@ alloc_rule(list rule_list, vector_t *strvec)
 				goto fwmark_err;
 
 			if (*end == '/') {
-				if (end[1] == '-')
+				if (isspace(end[1]) || end[1] == '-')
 					goto fwmark_err;
-					
-				val1 = strtoul(end+1, &end, 0);
+
+				val1 = strtoul(end + 1, &end, 0);
 				if (val1 > UINT32_MAX)
 					goto fwmark_err;
 				new->mask |= IPRULE_BIT_FWMASK;
@@ -446,7 +609,7 @@ alloc_rule(list rule_list, vector_t *strvec)
 			if (true) {
 			} else {
 fwmark_err:
-				log_message(LOG_INFO, "Invalid rule fwmark %s specified", str);
+				report_config_error(CONFIG_GENERAL_ERROR, "Invalid rule fwmark %s specified", str);
 				new->mask &= (uint32_t)~IPRULE_BIT_FWMASK;
 				goto err;
 			}
@@ -454,7 +617,7 @@ fwmark_err:
 		else if (!strcmp(str, "realms")) {
 			str = strvec_slot(strvec, ++i);
 			if (get_realms(&uval32, str)) {
-				log_message(LOG_INFO, "invalid realms %s for rule", FMT_STR_VSLOT(strvec, i));
+				report_config_error(CONFIG_GENERAL_ERROR, "invalid realms %s for rule", FMT_STR_VSLOT(strvec, i));
 				goto err;
 			}
 
@@ -463,27 +626,24 @@ fwmark_err:
 			if (family == AF_UNSPEC)
 				family = AF_INET;
 			else if (family != AF_INET) {
-				log_message(LOG_INFO, "realms is only valid for IPv4");
+				report_config_error(CONFIG_GENERAL_ERROR, "realms is only valid for IPv4");
 				goto err;
 			}
 		}
 #if HAVE_DECL_FRA_SUPPRESS_PREFIXLEN
 		else if (!strcmp(str, "suppress_prefixlength") || !strcmp(str, "sup_pl")) {
-			str = strvec_slot(strvec, ++i);
-			val = strtoul(str, &end, 0);
-			if (*end || val > INT32_MAX) {
-				log_message(LOG_INFO, "Invalid suppress_prefixlength %s specified", str);
+			if (!read_unsigned_strvec(strvec, ++i, &val_unsigned, 0, INT32_MAX, false)) {
+				report_config_error(CONFIG_GENERAL_ERROR, "Invalid suppress_prefixlength %s specified", str);
 				goto err;
 			}
-			new->suppress_prefix_len = (uint32_t)val;
-			new->mask |= IPRULE_BIT_SUP_PREFIXLEN;
+			new->suppress_prefix_len = (int32_t)val_unsigned;
 			table_option = true;
 		}
 #endif
 #if HAVE_DECL_FRA_SUPPRESS_IFGROUP
 		else if (!strcmp(str, "suppress_ifgroup") || !strcmp(str, "sup_group")) {
 			if (!find_rttables_group(strvec_slot(strvec, ++i), &uval32)) {
-				log_message(LOG_INFO, "suppress_group %s is invalid", FMT_STR_VSLOT(strvec, i));
+				report_config_error(CONFIG_GENERAL_ERROR, "suppress_group %s is invalid", FMT_STR_VSLOT(strvec, i));
 				goto err;
 			}
 			new->suppress_group = uval32;
@@ -493,9 +653,9 @@ fwmark_err:
 #endif
 		else if (!strcmp(str, "dev") || !strcmp(str, "iif")) {
 			str = strvec_slot(strvec, ++i);
-			ifp = if_get_by_ifname(str);
+			ifp = if_get_by_ifname(str, IF_CREATE_IF_DYNAMIC);
 			if (!ifp) {
-				log_message(LOG_INFO, "Unknown interface %s for rule",  str);
+				report_config_error(CONFIG_GENERAL_ERROR, "WARNING - interface %s for rule doesn't exist",  str);
 				goto err;
 			}
 			new->iif = ifp;
@@ -503,9 +663,9 @@ fwmark_err:
 #if HAVE_DECL_FRA_OIFNAME
 		else if (!strcmp(str, "oif")) {
 			str = strvec_slot(strvec, ++i);
-			ifp = if_get_by_ifname(str);
+			ifp = if_get_by_ifname(str, IF_CREATE_IF_DYNAMIC);
 			if (!ifp) {
-				log_message(LOG_INFO, "Unknown interface %s for rule",  str);
+				report_config_error(CONFIG_GENERAL_ERROR, "WARNING - interface %s for rule doesn't exist",  str);
 				goto err;
 			}
 			new->oif = ifp;
@@ -514,13 +674,101 @@ fwmark_err:
 #if HAVE_DECL_FRA_TUN_ID
 		else if (!strcmp(str, "tunnel-id")) {
 			uint64_t val64;
-			val64 = strtoull(strvec_slot(strvec, ++i), &end, 0);
-			if (*end) {
-				log_message(LOG_INFO, "Invalid tunnel-id %s specified", str);
+			if (!read_unsigned64_strvec(strvec, ++i, &val64, 0, UINT64_MAX, false)) {
+				report_config_error(CONFIG_GENERAL_ERROR, "Invalid tunnel-id %s specified", str);
 				goto err;
 			}
 			new->tunnel_id = val64;
 		}
+#endif
+#if HAVE_DECL_FRA_UID_RANGE
+		else if (!strcmp(str, "uidrange")) {
+			uint32_t start, end;
+			if (sscanf(strvec_slot(strvec, ++i), "%" PRIu32 "-%" PRIu32, &start, &end) != 2) {
+				report_config_error(CONFIG_GENERAL_ERROR, "Invalid uidrange %s specified", str);
+				goto err;
+			}
+			new->mask |= IPRULE_BIT_UID_RANGE;
+			new->uid_range.start = start;
+			new->uid_range.end = end;
+		}
+#endif
+#if HAVE_DECL_FRA_L3MDEV
+		else if (!strcmp(str, "l3mdev")) {
+			new->l3mdev = true;
+			if (new->action != FR_ACT_UNSPEC) {
+				report_config_error(CONFIG_GENERAL_ERROR, "Cannot specify l3mdev with other action");
+				goto err;
+			}
+			new->action = FR_ACT_TO_TBL;
+		}
+#endif
+#if HAVE_DECL_FRA_PROTOCOL
+		else if (!strcmp(str, "protocol")) {
+			if (!read_unsigned_strvec(strvec, ++i, &val_unsigned, 0, UINT8_MAX, false))
+				report_config_error(CONFIG_GENERAL_ERROR, "Invalid protocol %s", FMT_STR_VSLOT(strvec, i));
+			else {
+				new->protocol = val_unsigned;
+				new->mask |= IPRULE_BIT_PROTOCOL;
+			}
+		}
+#endif
+#if HAVE_DECL_FRA_IP_PROTO
+		else if (!strcmp(str, "ipproto")) {
+			int ip_proto = inet_proto_a2n(strvec_slot(strvec, ++i));
+			if (ip_proto < 0 || ip_proto > UINT8_MAX)
+				report_config_error(CONFIG_GENERAL_ERROR, "Invalid ipproto %s", FMT_STR_VSLOT(strvec, i));
+			else {
+				new->ip_proto = ip_proto;
+				new->mask |= IPRULE_BIT_IP_PROTO;
+			}
+		}
+#endif
+#if HAVE_DECL_FRA_SPORT_RANGE
+		else if (!strcmp(str, "sport")) {
+			struct fib_rule_port_range sport;
+			int ret;
+
+			ret = sscanf(strvec_slot(strvec, ++i), "%hu-%hu", &sport.start, &sport.end);
+			if (ret == 1)
+				sport.end = sport.start;
+			if (ret != 2)
+				report_config_error(CONFIG_GENERAL_ERROR, "invalid sport range %s", FMT_STR_VSLOT(strvec, i));
+			else {
+				new->src_port = sport;
+				new->mask |= IPRULE_BIT_SPORT_RANGE;
+			}
+		}
+#endif
+#if HAVE_DECL_FRA_DPORT_RANGE
+		else if (!strcmp(str, "dport")) {
+			struct fib_rule_port_range dport;
+			int ret;
+
+			ret = sscanf(strvec_slot(strvec, ++i), "%hu-%hu", &dport.start, &dport.end);
+			if (ret == 1)
+				dport.end = dport.start;
+			if (ret != 2)
+				report_config_error(CONFIG_GENERAL_ERROR, "invalid dport range %s", FMT_STR_VSLOT(strvec, i));
+			else {
+				new->dst_port = dport;
+				new->mask |= IPRULE_BIT_DPORT_RANGE;
+			}
+		}
+#endif
+
+		else if (!strcmp(str, "no_track"))
+			new->dont_track = true;
+#if HAVE_DECL_FRA_OIFNAME
+		else if (allow_track_group && !strcmp(str, "track_group")) {
+			i++;
+			if (new->track_group) {
+				report_config_error(CONFIG_GENERAL_ERROR, "track_group %s is a duplicate", FMT_STR_VSLOT(strvec, i));
+				break;
+			}
+			if (!(new->track_group = find_track_group(strvec_slot(strvec, i))))
+                                report_config_error(CONFIG_GENERAL_ERROR, "track_group %s not found", FMT_STR_VSLOT(strvec, i));
+		}
 #endif
 		else {
 			uint8_t action = FR_ACT_UNSPEC;
@@ -529,12 +777,11 @@ fwmark_err:
 				str = strvec_slot(strvec, ++i);
 
 			if (!strcmp(str, "goto")) {
-				val = strtoul(strvec_slot(strvec, ++i), &end, 0);
-				if (*end || val > UINT32_MAX) {
-					log_message(LOG_INFO, "Invalid target %s specified", str);
+				if (!read_unsigned_strvec(strvec, ++i, &val_unsigned, 0, UINT32_MAX, false)) {
+					report_config_error(CONFIG_GENERAL_ERROR, "Invalid target %s specified", str);
 					goto err;
 				}
-				new->goto_target = (uint32_t)val;
+				new->goto_target = (uint32_t)val_unsigned;
 				action = FR_ACT_GOTO;
 			}
 			else if (!strcmp(str, "nop")) {
@@ -548,16 +795,16 @@ fwmark_err:
 				else if (action == RTN_PROHIBIT)
 					action = FR_ACT_PROHIBIT;
 				else {
-					log_message(LOG_INFO, "Invalid rule action %s", str);
+					report_config_error(CONFIG_GENERAL_ERROR, "Invalid rule action %s", str);
 					goto err;
 				}
 			}
 			else {
-				log_message(LOG_INFO, "Unknown rule option %s", str);
+				report_config_error(CONFIG_GENERAL_ERROR, "Unknown rule option %s", str);
 				goto err;
 			}
 			if (new->action != FR_ACT_UNSPEC) {
-				log_message(LOG_INFO, "Cannot specify more than one of table/nop/goto/blackhole/prohibit/unreachable for rule");
+				report_config_error(CONFIG_GENERAL_ERROR, "Cannot specify more than one of table/nop/goto/blackhole/prohibit/unreachable/l3mdev for rule");
 				goto err;
 			}
 			new->action = action;
@@ -565,28 +812,67 @@ fwmark_err:
 		i++;
 	}
 
-	if (new->action == FR_ACT_GOTO &&
-	    new->mask & IPRULE_BIT_PRIORITY &&
-	    new->priority >= new->goto_target)
-	{
-		log_message(LOG_INFO, "Invalid rule - preference %u >= goto target %u", new->priority, new->goto_target);
-		goto err;
+	if (new->action == FR_ACT_GOTO) {
+		if (new->mask & IPRULE_BIT_PRIORITY) {
+			if (new->priority >= new->goto_target) {
+				report_config_error(CONFIG_GENERAL_ERROR, "Invalid rule - preference %u >= goto target %u", new->priority, new->goto_target);
+				goto err;
+			}
+		} else {
+			report_config_error(CONFIG_GENERAL_ERROR, "Invalid rule - goto target %u specified without preference", new->goto_target);
+			goto err;
+		}
 	}
 
 	if (new->action == FR_ACT_UNSPEC) {
-		log_message(LOG_INFO, "No action specified for rule - ignoring");
+		report_config_error(CONFIG_GENERAL_ERROR, "No action specified for rule - ignoring");
 		goto err;
 	}
 
 	if (new->action != FR_ACT_TO_TBL && table_option) {
-		log_message(LOG_INFO, "suppressor/realm specified for non table action - skipping");
+		report_config_error(CONFIG_GENERAL_ERROR, "suppressor/realm specified for non table action - skipping");
 		goto err;
 	}
 
+#if HAVE_DECL_FRA_L3MDEV
+	if (new->table && new->l3mdev) {
+		report_config_error(CONFIG_GENERAL_ERROR, "table cannot be specified for l3mdev rules");
+		goto err;
+	}
+#endif
+
+#if HAVE_DECL_FRA_PROTOCOL
+	if (!new->dont_track) {
+		if ((new->mask & IPRULE_BIT_PROTOCOL) && new->protocol != RTPROT_KEEPALIVED)
+			report_config_error(CONFIG_GENERAL_ERROR, "Rule cannot be tracked if protocol is not RTPROT_KEEPALIVED(%d), resetting protocol", RTPROT_KEEPALIVED);
+		new->protocol = RTPROT_KEEPALIVED;
+		new->mask |= IPRULE_BIT_PROTOCOL;
+	}
+#endif
+
+	if (new->track_group && !new->iif) {
+		report_config_error(CONFIG_GENERAL_ERROR, "Static rule cannot have track_group if dev/iif not specified");
+		new->track_group = NULL;
+	}
+
+	new->family = (family == AF_UNSPEC) ? AF_INET : family;
+	if (new->to_addr && new->to_addr->ifa.ifa_family == AF_UNSPEC)
+		new->to_addr->ifa.ifa_family = new->family;
+	if (new->from_addr && new->from_addr->ifa.ifa_family == AF_UNSPEC)
+		new->from_addr->ifa.ifa_family = new->family;
+
+	if (!(new->mask & IPRULE_BIT_PRIORITY)) {
+		new->priority = new->family == AF_INET ? next_rule_priority_ipv4-- : next_rule_priority_ipv6--;
+		new->mask |= IPRULE_BIT_PRIORITY;
+		report_config_error(CONFIG_GENERAL_ERROR, "Rule has no preference specifed - setting to %u. This is probably not what you want.", new->priority);
+	}
+
 	list_add(rule_list, new);
 	return;
 
 err:
+	FREE_PTR(new->to_addr);
+	FREE_PTR(new->from_addr);
 	FREE_PTR(new);
 }
 
@@ -641,3 +927,10 @@ clear_diff_srules(void)
 {
 	clear_diff_rules(old_vrrp_data->static_rules, vrrp_data->static_rules);
 }
+
+void
+reset_next_rule_priority(void)
+{
+	next_rule_priority_ipv4 = RULE_START_PRIORITY;
+	next_rule_priority_ipv6 = RULE_START_PRIORITY;
+}
