@@ -19,18 +19,21 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #include "config.h"
 
+#include <openssl/md5.h>
+#include <string.h>
+
 #include "vrrp_ipsecah.h"
 
 #define	BLOCK_SIZE	64
 
 /* hmac_md5 computation according to the RFCs 2085 & 2104 */
 void
-hmac_md5(unsigned char *buffer, int buffer_len, unsigned char *key, int key_len,
+hmac_md5(unsigned char *buffer, size_t buffer_len, unsigned char *key, size_t key_len,
 	 unsigned char *digest)
 {
 	MD5_CTX context;
