@@ -19,7 +19,7 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2016 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 /* The way iptables appears to work is that when we do an iptc_init, we get a
@@ -53,12 +53,18 @@
 #include <net/if.h>
 #endif
 
-#include <libiptc/libiptc.h>
+#ifdef _HAVE_LIBIPTC_
+#include <libiptc/libxtc.h>
+#endif
 #include <stdint.h>
 #ifdef _HAVE_LIBIPSET_
 #include <xtables.h>
+#ifdef USE_LIBIPSET_LINUX_IP_SET_H
+#include <libipset/linux_ip_set.h>
+#else
 #include <linux/netfilter/ipset/ip_set.h>
 #endif
+#endif
 #include <sys/stat.h>
 #include <sys/vfs.h>
 #include <linux/magic.h>
@@ -70,8 +76,8 @@
 #include "vrrp_ipset.h"
 #endif
 #include "logger.h"
-#include "memory.h"
 #include "global_data.h"
+#include "memory.h"
 
 struct ipt_handle {
 	struct iptc_handle *h4;
@@ -462,7 +468,7 @@ iptables_init_lib(void)
 #endif
 
 #ifdef _HAVE_LIBIPSET_
-	if (global_data->using_ipsets && !ipset_init())
+	if (global_data->using_ipsets && !ipset_initialise())
 		global_data->using_ipsets = false;
 #endif
 
