@@ -17,13 +17,16 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@linux-vs.org>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #ifndef _BITOPS_H
 #define _BITOPS_H
 
+#include "config.h"
+
 #include <limits.h>
+#include <stdbool.h>
 
 /* Defines */
 #define BIT_PER_LONG	(CHAR_BIT * sizeof(unsigned long))
@@ -41,7 +44,7 @@ static inline void __clear_bit(unsigned idx, unsigned long *bmap)
 	bmap[BIT_WORD(idx)] &= ~BIT_MASK(idx);
 }
 
-static inline bool __test_bit(unsigned idx, unsigned long *bmap)
+static inline bool __test_bit(unsigned idx, const unsigned long *bmap)
 {
 	return !!(bmap[BIT_WORD(idx)] & BIT_MASK(idx));
 }
@@ -49,6 +52,7 @@ static inline bool __test_bit(unsigned idx, unsigned long *bmap)
 /* Bits */
 enum global_bits {
 	LOG_CONSOLE_BIT,
+	NO_SYSLOG_BIT,
 	DONT_FORK_BIT,
 	DUMP_CONF_BIT,
 #ifdef _WITH_VRRP_
@@ -59,14 +63,18 @@ enum global_bits {
 	DONT_RELEASE_IPVS_BIT,
 #endif
 	LOG_DETAIL_BIT,
+	LOG_EXTRA_DETAIL_BIT,
 	DONT_RESPAWN_BIT,
+#ifdef _MEM_CHECK_
 	MEM_ERR_DETECT_BIT,
 #ifdef _MEM_CHECK_LOG_
 	MEM_CHECK_LOG_BIT,
 #endif
+#endif
 #ifdef _WITH_LVS_
 	LOG_ADDRESS_CHANGES,
 #endif
+	CONFIG_TEST_BIT,
 };
 
 #endif
