@@ -17,13 +17,16 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@linux-vs.org>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #ifndef _HTML_H
 #define _HTML_H
 
+#include <sys/types.h>
+
 /* Prototypes */
+extern size_t extract_content_length(char *buffer, size_t size);
 extern int extract_status_code(char *buffer, size_t size);
 extern char *extract_html(char *buffer, size_t size_buffer);
 
