@@ -2,14 +2,13 @@
  * Soft:        Keepalived is a failover program for the LVS project
  *              <www.linuxvirtualserver.org>. It monitor & manipulate
  *              a loadbalanced server pool using multi-layer checks.
+ *		In addition it provides a VRRPv2 & VRRPv3 stack.
  *
- * Part:        list_head. This code is comming from Linux Kernel.
+ * Author:      Alexandre Cassen, <acassen@corp.free.fr>
  *
- * Author:      Alexandre Cassen, <acassen@linux-vs.org>
- *
- *              This program is distributed in the hope that it will be useful, 
- *              but WITHOUT ANY WARRANTY; without even the implied warranty of 
- *              MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
+ *              This program is distributed in the hope that it will be useful,
+ *              but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *              MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *              See the GNU General Public License for more details.
  *
  *              This program is free software; you can redistribute it and/or
@@ -17,7 +16,7 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@linux-vs.org>
+ * Copyright (C) 2018 Alexandre Cassen, <acassen@corp.free.fr>
  */
 
 #ifndef _LIST_HEAD_H
@@ -64,7 +63,8 @@ typedef struct list_head {
 
 #define LIST_HEAD_INIT(name) { &(name), &(name) }
 
-#define LIST_HEAD(name) \
+// TODO
+#define LH_LIST_HEAD(name) \
 	struct list_head name = LIST_HEAD_INIT(name)
 
 #define INIT_LIST_HEAD(ptr) do { \
@@ -100,6 +100,7 @@ static inline void list_head_add(struct list_head *new, struct list_head *head)
 	__list_add(new, head, head->next);
 }
 
+// list_head_add_tail
 /**
  * list_add_tail - add a new entry
  * @new: new entry to be added
@@ -293,6 +294,7 @@ void list_sort(struct list_head *head,
 	for (pos = (head)->next; pos != (head); \
 		pos = pos->next)
 
+// __list_for_each isn't different
 /**
  * __list_for_each	-	iterate over a list
  * @pos:	the &struct list_head to use as a loop counter.
@@ -404,6 +406,7 @@ void list_sort(struct list_head *head,
  * @head:	the head for your list.
  * @member:	the name of the list_struct within the struct.
  */
+// Try the alternative
 #define list_for_each_entry_safe(pos, n, head, member)			\
 	for (pos = list_entry((head)->next, typeof(*pos), member),	\
 		n = list_entry(pos->member.next, typeof(*pos), member);	\
@@ -418,13 +421,13 @@ void list_sort(struct list_head *head,
  * You lose the ability to access the tail in O(1).
  */
 
-struct hlist_head {
+typedef struct hlist_head {
 	struct hlist_node *first;
-};
+} hlist_head_t;
 
-struct hlist_node {
+typedef struct hlist_node {
 	struct hlist_node *next, **pprev;
-};
+} hlist_node_t;
 
 #define HLIST_HEAD_INIT { .first = NULL }
 #define HLIST_HEAD(name) struct hlist_head name = {  .first = NULL }
