@@ -18,21 +18,25 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@linux-vs.org>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #ifndef _MEMORY_H
 #define _MEMORY_H
 
+#include "config.h"
+
 /* system includes */
+#include <stddef.h>
+#ifdef _MEM_CHECK_
+#include <sys/types.h>
+#else
 #include <stdlib.h>
-#include <stdbool.h>
+#endif
 
 /* Local defines */
 #ifdef _MEM_CHECK_
 
-#define MAX_ALLOC_LIST 2048
-
 #define MALLOC(n)    ( keepalived_malloc((n), \
 		      (__FILE__), (char *)(__FUNCTION__), (__LINE__)) )
 #define FREE(b)      ( keepalived_free((b), \
@@ -44,12 +48,14 @@
 extern size_t mem_allocated;
 
 /* Memory debug prototypes defs */
-extern void *keepalived_malloc(size_t, char *, char *, int)
+extern void memcheck_log(const char *, const char *, const char *, const char *, int);
+extern void *keepalived_malloc(size_t, const char *, const char *, int)
 		__attribute__((alloc_size(1))) __attribute__((malloc));
-extern int keepalived_free(void *, char *, char *, int);
-extern void *keepalived_realloc(void *, size_t, char *, char *, int)
+extern void keepalived_free(void *, const char *, const char *, int);
+extern void *keepalived_realloc(void *, size_t, const char *, const char *, int)
 		__attribute__((alloc_size(2)));
 
+extern void keepalived_alloc_dump(void);
 extern void mem_log_init(const char *, const char *);
 extern void skip_mem_dump(void);
 extern void enable_mem_log_termination(void);
@@ -65,5 +71,6 @@ extern void *zalloc(unsigned long size);
 #endif
 
 /* Common defines */
+#define PMALLOC(p)	{ p = MALLOC(sizeof(*p)); }
 #define FREE_PTR(p)	{ if (p) { FREE(p);} }
 #endif
