@@ -18,19 +18,16 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2016 Alexandre Cassen, <acassen@linux-vs.org>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 #include "config.h"
 
-#include <stdint.h>
-#include <stdbool.h>
 #include <errno.h>
-
-#include <linux/socket.h>
+#include <stdlib.h>
+#include <string.h>
 #include <sys/socket.h>
 #include <linux/rtnetlink.h>
 
-#include "vector.h"
 #include "list.h"
 #include "memory.h"
 #include "logger.h"
@@ -39,14 +36,16 @@
 
 #define IPROUTE2_DIR	"/etc/iproute2/"
 
+#ifdef _HAVE_FIB_ROUTING_
 #define RT_TABLES_FILE	IPROUTE2_DIR "rt_tables"
 #define	RT_DSFIELD_FILE IPROUTE2_DIR "rt_dsfield"
 #define	RT_REALMS_FILE	IPROUTE2_DIR "rt_realms"
-#define	RT_SCOPES_FILE	IPROUTE2_DIR "rt_scopes"
 #define	RT_PROTOS_FILE	IPROUTE2_DIR "rt_protos"
 #if HAVE_DECL_FRA_SUPPRESS_IFGROUP
 #define	RT_GROUPS_FILE	IPROUTE2_DIR "group"
 #endif
+#endif
+#define	RT_SCOPES_FILE	IPROUTE2_DIR "rt_scopes"
 
 struct rt_entry {
 	unsigned int id;
@@ -54,6 +53,7 @@ struct rt_entry {
 } ;
 typedef struct rt_entry rt_entry_t;
 
+#ifdef _HAVE_FIB_ROUTING_
 static rt_entry_t rtntypes[] = {
 	{ RTN_LOCAL, "local"},
 	{ RTN_NAT, "nat"},
@@ -92,6 +92,14 @@ static rt_entry_t rtprot_default[] = {
 	{ 0, NULL},
 };
 
+static rt_entry_t rttable_default[] = {
+	{ RT_TABLE_DEFAULT, "default"},
+	{ RT_TABLE_MAIN, "main"},
+	{ RT_TABLE_LOCAL, "local"},
+	{ 0, NULL},
+};
+#endif
+
 static rt_entry_t rtscope_default[] = {
 	{ RT_SCOPE_UNIVERSE, "global"},
 	{ RT_SCOPE_NOWHERE, "nowhere"},
@@ -101,15 +109,9 @@ static rt_entry_t rtscope_default[] = {
 	{ 0, NULL},
 };
 
-static rt_entry_t rttable_default[] = {
-	{ RT_TABLE_DEFAULT, "default"},
-	{ RT_TABLE_MAIN, "main"},
-	{ RT_TABLE_LOCAL, "local"},
-	{ 0, NULL},
-};
-
 #define	MAX_RT_BUF	128
 
+#ifdef _HAVE_FIB_ROUTING_
 static list rt_tables;
 static list rt_dsfields;
 #if HAVE_DECL_FRA_SUPPRESS_IFGROUP
@@ -117,28 +119,11 @@ static list rt_groups;
 #endif
 static list rt_realms;
 static list rt_protos;
+#endif
 static list rt_scopes;
 
 static char ret_buf[11];	/* uint32_t in decimal */
 
-static void
-free_rt_entry(void *e)
-{
-	rt_entry_t *rte = (rt_entry_t*)e;
-
-	if (rte->name)
-		FREE(rte->name);
-	FREE(rte);
-}
-
-static void
-dump_rt_entry(void *e)
-{
-	rt_entry_t *rte = (rt_entry_t *)e;
-
-	log_message(LOG_INFO, "rt_table %u, name %s", rte->id, rte->name);
-}
-
 static void
 read_file(const char* file_name, list *l, uint32_t max)
 {
@@ -147,6 +132,8 @@ read_file(const char* file_name, list *l, uint32_t max)
 	vector_t *strvec = NULL;
 	char buf[MAX_RT_BUF];
 	unsigned long id;
+	char *number;
+	char *endptr;
 
 	fp = fopen(file_name, "r");
 	if (!fp)
@@ -169,8 +156,10 @@ read_file(const char* file_name, list *l, uint32_t max)
 			goto err;
 		}
 
-		id = strtoul(FMT_STR_VSLOT(strvec, 0), NULL, 0);
-		if (id > max) {
+		number = strvec_slot(strvec, 0);
+		number += strspn(number, " \t");
+		id = strtoul(number, &endptr, 0);
+		if (*number == '-' || number == endptr || *endptr || id > max) {
 			FREE(rte);
 			free_strvec(strvec);
 			continue;
@@ -208,6 +197,7 @@ err:
 void
 clear_rt_names(void)
 {
+#ifdef _HAVE_FIB_ROUTING_
 	free_list(&rt_tables);
 	free_list(&rt_dsfields);
 #if HAVE_DECL_FRA_SUPPRESS_IFGROUP
@@ -215,9 +205,28 @@ clear_rt_names(void)
 #endif
 	free_list(&rt_realms);
 	free_list(&rt_protos);
+#endif
 	free_list(&rt_scopes);
 }
 
+static void
+free_rt_entry(void *e)
+{
+	rt_entry_t *rte = (rt_entry_t*)e;
+
+	if (rte->name)
+		FREE(rte->name);
+	FREE(rte);
+}
+
+static void
+dump_rt_entry(FILE *fp, void *e)
+{
+	rt_entry_t *rte = (rt_entry_t *)e;
+
+	conf_write(fp, "rt_table %u, name %s", rte->id, rte->name);
+}
+
 static void
 add_default(list *l, const struct rt_entry* default_list)
 {
@@ -298,6 +307,7 @@ find_entry(const char *name, unsigned int *id, list *l, const char* file_name, c
 	return false;
 }
 
+#ifdef _HAVE_FIB_ROUTING_
 bool
 find_rttables_table(const char *name, uint32_t *id)
 {
@@ -309,7 +319,7 @@ find_rttables_dsfield(const char *name, uint8_t *id)
 {
 	uint32_t val;
 	bool ret;
-	
+
 	ret = find_entry(name, &val, &rt_dsfields, RT_DSFIELD_FILE, NULL, 255);
 	*id = val & 0xff;
 
@@ -335,25 +345,13 @@ find_rttables_proto(const char *name, uint8_t *id)
 {
 	uint32_t val;
 	bool ret;
-	
+
 	ret = find_entry(name, &val, &rt_protos, RT_PROTOS_FILE, rtprot_default, 255);
 	*id = val & 0xff;
 
 	return ret;
 }
 
-bool
-find_rttables_scope(const char *name, uint8_t *id)
-{
-	uint32_t val;
-	bool ret;
-	
-	ret = find_entry(name, &val, &rt_scopes, RT_SCOPES_FILE, rtscope_default, 255);
-	*id = val & 0xff;
-
-	return ret;
-}
-
 bool
 find_rttables_rtntype(const char *str, uint8_t *id)
 {
@@ -369,12 +367,13 @@ find_rttables_rtntype(const char *str, uint8_t *id)
 	}
 
 	res = strtoul(str, &end, 0);
-	if (*end || res > 255)
+	if (*end || res > 255 || str[0] == '-')
 		return false;
 
 	*id = (uint8_t)res;
 	return true;
 }
+#endif
 
 static const char *
 get_entry(unsigned int id, list* l, const char* file_name, const struct rt_entry* default_list, uint32_t max)
@@ -397,12 +396,7 @@ get_entry(unsigned int id, list* l, const char* file_name, const struct rt_entry
 	return ret_buf;
 }
 
-const char *
-get_rttables_scope(uint32_t id)
-{
-	return get_entry(id, &rt_scopes, RT_SCOPES_FILE, rtscope_default, 255);
-}
-
+#ifdef _HAVE_FIB_ROUTING_
 #if HAVE_DECL_FRA_SUPPRESS_IFGROUP
 const char *
 get_rttables_group(uint32_t id)
@@ -424,3 +418,22 @@ get_rttables_rtntype(uint8_t val)
 	snprintf(ret_buf, sizeof(ret_buf), "%u", val);
 	return ret_buf;
 }
+#endif
+
+bool
+find_rttables_scope(const char *name, uint8_t *id)
+{
+	uint32_t val;
+	bool ret;
+
+	ret = find_entry(name, &val, &rt_scopes, RT_SCOPES_FILE, rtscope_default, 255);
+	*id = val & 0xff;
+
+	return ret;
+}
+
+const char *
+get_rttables_scope(uint32_t id)
+{
+	return get_entry(id, &rt_scopes, RT_SCOPES_FILE, rtscope_default, 255);
+}
