@@ -17,7 +17,7 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@linux-vs.org>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #include "config.h"
@@ -25,12 +25,46 @@
 #include "vector.h"
 #include "memory.h"
 
+/* Function to call if attempt to read beyond end of strvec */
+static null_strvec_handler_t null_strvec_handler;
+
+null_strvec_handler_t register_null_strvec_handler(null_strvec_handler_t null_strvec_func)
+{
+	null_strvec_handler_t old_handler = null_strvec_handler;
+
+	null_strvec_handler = null_strvec_func;
+
+	return old_handler;
+}
+
+null_strvec_handler_t unregister_null_strvec_handler(void)
+{
+	null_strvec_handler_t old_handler = null_strvec_handler;
+
+	null_strvec_handler = NULL;
+
+	return old_handler;
+}
+
+void *strvec_slot(const vector_t *strvec, size_t index)
+{
+	if (strvec &&
+	    index < vector_size(strvec) &&
+	    strvec->slot[index])
+		return strvec->slot[index];
+
+	if (null_strvec_handler)
+		(*null_strvec_handler)(strvec, index);
+
+	return "";
+}
+
 /*
  * Initialize vector struct.
  * allocalted 'size' slot elements then return vector.
  */
 vector_t *
-vector_alloc(void)
+vector_alloc_r(void)
 {
 	vector_t *v = (vector_t *) MALLOC(sizeof(vector_t));
 	return v;
@@ -55,7 +89,7 @@ vector_init(unsigned int size)
 
 /* allocated one slot */
 void
-vector_alloc_slot(vector_t *v)
+vector_alloc_slot_r(vector_t *v)
 {
 	v->allocated += VECTOR_DEFAULT_SIZE;
 	if (v->slot)
@@ -249,12 +283,14 @@ vector_only_index_free(void *slot)
 #endif
 
 void
-vector_free(vector_t *v)
+vector_free_r(vector_t *v)
 {
-	FREE(v->slot);
+	if (v->slot)
+		FREE(v->slot);
 	FREE(v);
 }
 
+#ifdef _INCLUDE_UNUSED_CODE_
 /* dump vector slots */
 void
 vector_dump(FILE *fp, vector_t *v)
@@ -269,6 +305,7 @@ vector_dump(FILE *fp, vector_t *v)
 		}
 	}
 }
+#endif
 
 /* String vector related */
 void
